from SpectrumCommon.Const.GB import *

# Database latest versions

RP_COUNTRY_DEFAULTS_DB_CURR_VERSION          = 'V4'
RP_COUNTRY_DEFAULTS_SOURCES_DB_CURR_VERSION  = 'V1'
RP_CONTRACEPTIVE_USE_MED_DB_CURR_VERSION     = 'V1'
RP_CONTRACEPTIVE_USE_ACC_DB_CURR_VERSION     = 'V1'
RP_SSP1_TEMPS_DB_CURR_VERSION                = 'V1'
RP_SSP2_TEMPS_DB_CURR_VERSION                = 'V1'
RP_SSP3_TEMPS_DB_CURR_VERSION                = 'V1'
RP_SSP5_TEMPS_DB_CURR_VERSION                = 'V1'
RP_URBAN_DB_CURR_VERSION                     = 'V1'

# Database names (also the names of the sheets in RPModData.xlsx)

RP_COUNTRY_DEFAULTS_DB_NAME         = 'CountryDefaultsDB'
RP_COUNTRY_DEFAULTS_SOURCES_DB_NAME = 'CountryDefaultsSourcesDB'
RP_CONTRACEPTIVE_USE_MED_DB_NAME    = 'ContraceptiveUseMedDB'
RP_CONTRACEPTIVE_USE_ACC_DB_NAME    = 'ContraceptiveUseAccDB'
RP_SSP1_TEMPS_DB_NAME               = 'SSP1TempsDB'
RP_SSP2_TEMPS_DB_NAME               = 'SSP2TempsDB'
RP_SSP3_TEMPS_DB_NAME               = 'SSP3TempsDB'
RP_SSP5_TEMPS_DB_NAME               = 'SSP5TempsDB'
RP_URBAN_DB_NAME                    = 'UrbanDB'

# Database directories for data by country. Files in these folders are named using the 3 letter iso country codes instead of the database names above.

RP_COUNTRY_DEFAULTS_DB_DIR          = r'CountryDefaults\countries'
RP_COUNTRY_DEFAULTS_SOURCES_DB_DIR  = r'CountryDefaultsSources\countries'
RP_CONTRACEPTIVE_USE_MED_DB_DIR     = r'ContraceptiveUseMed\countries'
RP_CONTRACEPTIVE_USE_ACC_DB_DIR     = r'ContraceptiveUseAcc\countries'
RP_SSP1_TEMPS_DB_DIR                = r'SSP1Temps\countries'
RP_SSP2_TEMPS_DB_DIR                = r'SSP2Temps\countries'
RP_SSP3_TEMPS_DB_DIR                = r'SSP3Temps\countries'
RP_SSP5_TEMPS_DB_DIR                = r'SSP5Temps\countries'
RP_URBAN_DB_DIR                     = r'Urban\countries'

# Resulting change (RC) shapes

RP_RC_BY_SEX_YEAR = 1
RP_RC_BY_YEAR = 2
RP_RC_NOT_APPLICABLE = 3

# #####################################################################################################
#                                                                                                     #
#                                       Sectors                                                       #
#                                                                                                     #
#######################################################################################################

RP_EDUCATION_SECTOR_CURR_ID      = 1
RP_HEALTH_SECTOR_CURR_ID         = 2
RP_WASH_SECTOR_CURR_ID           = 3
RP_FOOD_SECURITY_SECTOR_CURR_ID  = 4
RP_URBANIZATION_SECTOR_CURR_ID   = 5
RP_ENERGY_SECTOR_CURR_ID         = 6
RP_ECONOMIC_SECTOR_CURR_ID       = 7
RP_BUILD_YOUR_OWN_SECTOR_CURR_ID = 8
RP_NUM_DEFAULT_SECTORS           = RP_BUILD_YOUR_OWN_SECTOR_CURR_ID

RP_EDUCATION_SECTOR_MST_ID      = 1
RP_HEALTH_SECTOR_MST_ID         = 2
RP_WASH_SECTOR_MST_ID           = 3
RP_FOOD_SECURITY_SECTOR_MST_ID  = 4
RP_URBANIZATION_SECTOR_MST_ID   = 5
RP_ENERGY_SECTOR_MST_ID         = 6
RP_ECONOMIC_SECTOR_MST_ID       = 7
RP_BUILD_YOUR_OWN_SECTOR_MST_ID = 8

RP_SECTOR_MST_IDS = [
    RP_EDUCATION_SECTOR_MST_ID,
    RP_HEALTH_SECTOR_MST_ID,
    RP_WASH_SECTOR_MST_ID,
    RP_FOOD_SECURITY_SECTOR_MST_ID,
    RP_URBANIZATION_SECTOR_MST_ID,
    RP_ENERGY_SECTOR_MST_ID,
    RP_ECONOMIC_SECTOR_MST_ID,
    RP_BUILD_YOUR_OWN_SECTOR_MST_ID,
]

RP_SECTOR_CURR_ID_MAP = {
    RP_EDUCATION_SECTOR_MST_ID      : RP_EDUCATION_SECTOR_CURR_ID,
    RP_HEALTH_SECTOR_MST_ID         : RP_HEALTH_SECTOR_CURR_ID,
    RP_WASH_SECTOR_MST_ID           : RP_WASH_SECTOR_CURR_ID,
    RP_FOOD_SECURITY_SECTOR_MST_ID  : RP_FOOD_SECURITY_SECTOR_CURR_ID,
    RP_URBANIZATION_SECTOR_MST_ID   : RP_URBANIZATION_SECTOR_CURR_ID,
    RP_ENERGY_SECTOR_MST_ID         : RP_ENERGY_SECTOR_CURR_ID,
    RP_ECONOMIC_SECTOR_MST_ID       : RP_ECONOMIC_SECTOR_CURR_ID,
    RP_BUILD_YOUR_OWN_SECTOR_MST_ID : RP_BUILD_YOUR_OWN_SECTOR_CURR_ID,
}

# #####################################################################################################
#                                                                                                     #
#                                       Goals                                                         #
#                                                                                                     #
#######################################################################################################

#######################################   Education Sector   ##########################################

RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID           = 1 # 1. Increase primary enrollment
RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID            = 2 # 2. Increase secondary enrollment 
RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID = 3 # 3. Improve the student-to-teacher ratio (primary)

RP_EDU_GOAL_MST_IDS = [
    RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID,
    RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID,
    RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_EDU_GOAL_RC_FORMAT = {
    RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID           : RP_RC_BY_SEX_YEAR,
    RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID            : RP_RC_BY_SEX_YEAR,
    RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID : RP_RC_BY_YEAR,
}

#########################################   Health Sector   ##########################################

RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID        = 1 # 1. Increase percentage of births attended by skilled personnel
RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID = 2 # 2. Increase percentage of pregnant women receiving 4+ antenatal care visits
RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID         = 3 # 3. Increase access to health professionals

RP_HLTH_GOAL_MST_IDS = [
    RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID,
    RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID,
    RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_HLTH_GOAL_RC_FORMAT = {
    RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID        : RP_RC_BY_YEAR,
    RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID : RP_RC_BY_YEAR,
    RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID         : RP_RC_BY_YEAR,
}

#########################################   WASH   ##########################################

RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID = 1 # 1. Increase access to an improved water source
RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID      = 2 # 2. Increase access to improved sanitation facility

RP_WASH_GOAL_MST_IDS = [
    RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID,
    RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_WASH_GOAL_RC_FORMAT = {
    RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID  : RP_RC_BY_YEAR,
    RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID       : RP_RC_BY_YEAR,
}

#########################################   Food security   ##########################################

RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID = 1 # 1. Increase proportion of domestic production of main staple crop
RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID              = 2 # 2. Reduce severe food insecurity

RP_FOOD_GOAL_MST_IDS = [
    RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID,
    RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_FOOD_GOAL_RC_FORMAT = {
    RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID : RP_RC_BY_YEAR,
    RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID              : RP_RC_BY_YEAR,
}

#########################################   Urbanization   ##########################################

RP_URB_1_SLOW_INCR_NUM_PEOPLE_GOAL_MST_ID   = 1 # 1. Slow increase in the number of people living in urban areas
RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID = 2 # 2. Increase access to urban housing: reduction of living in informal settlements

RP_URB_GOAL_MST_IDS = [
    RP_URB_1_SLOW_INCR_NUM_PEOPLE_GOAL_MST_ID,
    RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_URB_GOAL_RC_FORMAT = {
    RP_URB_1_SLOW_INCR_NUM_PEOPLE_GOAL_MST_ID    : RP_RC_NOT_APPLICABLE,
    RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID  : RP_RC_BY_YEAR,
}

#########################################   Energy   ##########################################

RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID          = 1 # 1. Increase population access to electricity
RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID = 2 # 2. Increase production of electricity to sufficient levels
RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID     = 3 # 3. Increase household access to clean cooking fuel and technologies

RP_ENERGY_GOAL_MST_IDS = [
    RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID,
    RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID,
    RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_ENERGY_GOAL_RC_FORMAT = {
    RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID           : RP_RC_BY_YEAR,
    RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID  : RP_RC_BY_YEAR,
    RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID      : RP_RC_BY_YEAR,
}

#########################################   Economic   ##########################################

RP_ECON_1_REDUCE_AGE_DEPEND_RATIO_GOAL_MST_ID = 1 # 1. Reduce age dependency ratio 
RP_ECON_2_INCR_GDP_PER_CAPITA_GOAL_MST_ID     = 2 # 2. Increase GDP per capita
RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID       = 3 # 3. Decrease unemployment 
RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID       = 4 # 4. Reduce proportion of youth not in education, employment, or training (NEET)

RP_ECON_GOAL_MST_IDS = [
    RP_ECON_1_REDUCE_AGE_DEPEND_RATIO_GOAL_MST_ID,
    RP_ECON_2_INCR_GDP_PER_CAPITA_GOAL_MST_ID,
    RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID,
    RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID
]

# Tells us the shape of the resulting change field for each goal.

RP_ECON_GOAL_RC_FORMAT = {
    RP_ECON_1_REDUCE_AGE_DEPEND_RATIO_GOAL_MST_ID  : RP_RC_NOT_APPLICABLE,
    RP_ECON_2_INCR_GDP_PER_CAPITA_GOAL_MST_ID      : RP_RC_NOT_APPLICABLE,
    RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID        : RP_RC_BY_YEAR,
    RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID        : RP_RC_BY_SEX_YEAR
}

#########################################   Build Your Own   ##########################################

RP_BYO_1_BYO_GOAL_MST_ID = 1 # 1. Reduce age dependency ratio 

RP_BYO_GOAL_MST_IDS = [
    RP_BYO_1_BYO_GOAL_MST_ID,
]

# Tells us the shape of the resulting change field for each goal.

RP_BYO_GOAL_RC_FORMAT = {
    RP_BYO_1_BYO_GOAL_MST_ID : RP_RC_BY_YEAR,
}

###################################################################################################

RP_SECTOR_GOAL_RC_FORMATS = {
    RP_EDUCATION_SECTOR_MST_ID       : RP_EDU_GOAL_RC_FORMAT,
    RP_HEALTH_SECTOR_MST_ID          : RP_HLTH_GOAL_RC_FORMAT,
    RP_WASH_SECTOR_MST_ID            : RP_WASH_GOAL_RC_FORMAT,
    RP_FOOD_SECURITY_SECTOR_MST_ID   : RP_FOOD_GOAL_RC_FORMAT,
    RP_URBANIZATION_SECTOR_MST_ID    : RP_URB_GOAL_RC_FORMAT,
    RP_ENERGY_SECTOR_MST_ID          : RP_ENERGY_GOAL_RC_FORMAT,
    RP_ECONOMIC_SECTOR_MST_ID        : RP_ECON_GOAL_RC_FORMAT,
    RP_BUILD_YOUR_OWN_SECTOR_MST_ID  : RP_BYO_GOAL_RC_FORMAT,
}

# #####################################################################################################
#                                                                                                     #
#                                       Subgoals                                                      #
#                                                                                                     #
#######################################################################################################

#######################################   Education Sector   ##########################################

# Education Sector, 1. Increase primary enrollment Goal

RP_EDU_1_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID   = 1 # 1. Universal primary education
RP_EDU_1_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID = 2 # 2. Achieve gender equality
RP_EDU_1_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID = 3 # 3. User-specified as percent of children

RP_EDU_1_SUBGOAL_MST_IDS = [
    RP_EDU_1_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID,
    RP_EDU_1_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID,
    RP_EDU_1_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID, 
]

# Education Sector, 2. Increase secondary enrollment Goal

RP_EDU_2_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID   = 1 # 1. Universal primary education
RP_EDU_2_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID = 2 # 2. Achieve gender equality
RP_EDU_2_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID = 3 # 3. User-specified as percent of children

RP_EDU_2_SUBGOAL_MST_IDS = [
    RP_EDU_2_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID,
    RP_EDU_2_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID,
    RP_EDU_2_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID, 
]

# Education Sector, 3. Improve the student-to-teacher ratio  Goal

RP_EDU_3_1_REC_RATIO_SUBG_MST_ID       = 1 # 1. Recommended ratio (25:1)
RP_EDU_3_2_USER_SPEC_RATIO_SUBG_MST_ID = 2 # 2. User specifies ratio

RP_EDU_3_SUBGOAL_MST_IDS = [
    RP_EDU_3_1_REC_RATIO_SUBG_MST_ID,
    RP_EDU_3_2_USER_SPEC_RATIO_SUBG_MST_ID,
]

RP_EDU_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID           : RP_EDU_1_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID,
    RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID            : RP_EDU_2_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID,
    RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID : RP_EDU_3_1_REC_RATIO_SUBG_MST_ID,
}

#########################################   Health Sector   ##########################################

# Health Sector, 1. Increase percentage of births attended by skilled personnel Goal

RP_HLTH_1_1_ALL_SUBG_MST_ID                    = 1 # 1. All
RP_HLTH_1_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID = 2 # 2. X percent based on Y targets
RP_HLTH_1_3_USER_SPEC_PERC_BIRTHS_SUBG_MST_ID  = 3 # 3. User-specified as percent of children

RP_HLTH_1_SUBGOAL_MST_IDS = [
    RP_HLTH_1_1_ALL_SUBG_MST_ID,
    RP_HLTH_1_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID,
    RP_HLTH_1_3_USER_SPEC_PERC_BIRTHS_SUBG_MST_ID, 
]

# Health Sector, 2. Increase percentage of pregnant women receiving 4+ antenatal care visits Goal

RP_HLTH_2_1_ALL_SUBG_MST_ID                        = 1 # 1. All
RP_HLTH_2_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID     = 2 # 2. X percent based on Y targets
RP_HLTH_2_3_USER_SPEC_PERC_PREGNANCIES_SUBG_MST_ID = 3 # 3. User-specified as percent of pregnancies

RP_HLTH_2_SUBGOAL_MST_IDS = [
    RP_HLTH_2_1_ALL_SUBG_MST_ID,
    RP_HLTH_2_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID,
    RP_HLTH_2_3_USER_SPEC_PERC_PREGNANCIES_SUBG_MST_ID, 
]

# Health Sector, 3. Increase access to health professionals Goal

RP_HLTH_3_1_WHO_REC_SUBG_MST_ID       = 1 # 1. WHO recommendation of 2.5 per 1,000
RP_HLTH_3_2_USER_SPEC_NUM_SUBG_MST_ID = 2 # 2. User-specified as # per 1,000

RP_HLTH_3_SUBGOAL_MST_IDS = [
    RP_HLTH_3_1_WHO_REC_SUBG_MST_ID,
    RP_HLTH_3_2_USER_SPEC_NUM_SUBG_MST_ID,
]

RP_HLTH_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID        : RP_HLTH_1_1_ALL_SUBG_MST_ID,
    RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID : RP_HLTH_2_1_ALL_SUBG_MST_ID,
    RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID         : RP_HLTH_3_1_WHO_REC_SUBG_MST_ID,
}

#########################################   WASH   ##########################################

RP_WASH_1_1_UNIVERSAL_ACCESS_SUBG_MST_ID  = 1 # 1. Universal access
RP_WASH_1_2_USER_SPEC_PERC_HH_SUBG_MST_ID = 2 # 2. User-specified as % of households

RP_WASH_1_SUBGOAL_MST_IDS = [
    RP_WASH_1_1_UNIVERSAL_ACCESS_SUBG_MST_ID,
    RP_WASH_1_2_USER_SPEC_PERC_HH_SUBG_MST_ID,
]

RP_WASH_2_1_UNIVERSAL_ACCESS_SUBG_MST_ID  = 1 # 1. Universal access
RP_WASH_2_2_USER_SPEC_PERC_HH_SUBG_MST_ID = 2 # 2. User-specified as % of households

RP_WASH_2_SUBGOAL_MST_IDS = [
    RP_WASH_2_1_UNIVERSAL_ACCESS_SUBG_MST_ID,
    RP_WASH_2_2_USER_SPEC_PERC_HH_SUBG_MST_ID,
]

RP_WASH_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID  : RP_WASH_1_1_UNIVERSAL_ACCESS_SUBG_MST_ID,
    RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID       : RP_WASH_2_1_UNIVERSAL_ACCESS_SUBG_MST_ID,
}

#########################################   Food security   ##########################################

RP_FOOD_1_1_ALL_DOMEST_PROD_SUBG_MST_ID          = 1 # 1. All domestically produced
RP_FOOD_1_2_USER_SPEC_PERC_DOM_GROWN_SUBG_MST_ID = 2 # 2. User-specified % domestically grown

RP_FOOD_1_SUBGOAL_MST_IDS = [
    RP_FOOD_1_1_ALL_DOMEST_PROD_SUBG_MST_ID,
    RP_FOOD_1_2_USER_SPEC_PERC_DOM_GROWN_SUBG_MST_ID,
]

RP_FOOD_2_1_ELIM_SEV_FOOD_INSECURITY_SUBG_MST_ID     = 1 # 1. Eliminate severe food insecurity
RP_FOOD_2_2_HALVE_SEV_FOOD_INSECURITY_SUBG_MST_ID    = 2 # 2. Halve severe food insecurity
RP_FOOD_2_3_USER_SPEC_PERC_FOOD_INSECURE_SUBG_MST_ID = 3 # 3. User-specified as % food insecure

RP_FOOD_2_SUBGOAL_MST_IDS = [
    RP_FOOD_2_1_ELIM_SEV_FOOD_INSECURITY_SUBG_MST_ID,
    RP_FOOD_2_2_HALVE_SEV_FOOD_INSECURITY_SUBG_MST_ID,
    RP_FOOD_2_3_USER_SPEC_PERC_FOOD_INSECURE_SUBG_MST_ID,
]

RP_FOOD_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID : RP_FOOD_1_1_ALL_DOMEST_PROD_SUBG_MST_ID,
    RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID              : RP_FOOD_2_1_ELIM_SEV_FOOD_INSECURITY_SUBG_MST_ID,
}

#########################################   Urbanization   ##########################################

# Nothing for Goal 1
RP_URB_1_SUBGOAL_MST_IDS = []

RP_URB_2_1_NO_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID    = 1 # 1. No population living in informal settlements
RP_URB_2_2_HALVE_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID = 2 # 2. Halve % of population living in informal settlements
RP_URB_2_3_USER_SPEC_PERC_URBAN_SUBG_MST_ID           = 3 # 3. User-specified % of urban in informal settlements

RP_URB_2_SUBGOAL_MST_IDS = [
    RP_URB_2_1_NO_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID,
    RP_URB_2_2_HALVE_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID,
    RP_URB_2_3_USER_SPEC_PERC_URBAN_SUBG_MST_ID,
]

RP_URB_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID : RP_URB_2_1_NO_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID,
}

#########################################   Energy   ##########################################

RP_ENERGY_1_1_UNIV_ACCESS_SUBG_MST_ID          = 1 # 1. Universal access (SDG 2030 goal)
RP_ENERGY_1_2_USER_SPEC_PERC_POP_SUBG_MST_ID   = 2 # 2. User-specified % of population

RP_ENERGY_1_SUBGOAL_MST_IDS = [
    RP_ENERGY_1_1_UNIV_ACCESS_SUBG_MST_ID,
    RP_ENERGY_1_2_USER_SPEC_PERC_POP_SUBG_MST_ID,
]

RP_ENERGY_2_1_1000_KWH_PER_PERSON_YEAR_MST_ID  = 1 # 1. 1000 kilowatt hour per person, per year
RP_ENERGY_2_2_USER_SPEC_PERC_POP_SUBG_MST_ID   = 2 # 2. User-specified % of population

RP_ENERGY_2_SUBGOAL_MST_IDS = [
    RP_ENERGY_2_1_1000_KWH_PER_PERSON_YEAR_MST_ID,
    RP_ENERGY_2_2_USER_SPEC_PERC_POP_SUBG_MST_ID,
]

RP_ENERGY_3_1_UNIV_ACCESS_SUBG_MST_ID          = 1 # 1. Universal access
RP_ENERGY_3_2_USER_SPEC_PERC_POP_SUBG_MST_ID   = 2 # 2. User-specified % of population

RP_ENERGY_3_SUBGOAL_MST_IDS = [
    RP_ENERGY_3_1_UNIV_ACCESS_SUBG_MST_ID,
    RP_ENERGY_3_2_USER_SPEC_PERC_POP_SUBG_MST_ID,
]

RP_ENERGY_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID           : RP_ENERGY_1_1_UNIV_ACCESS_SUBG_MST_ID,
    RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID  : RP_ENERGY_2_1_1000_KWH_PER_PERSON_YEAR_MST_ID,
    RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID      : RP_ENERGY_3_1_UNIV_ACCESS_SUBG_MST_ID,
}

#########################################   Economic   ##########################################

# Nothing for Goals 1, 2

RP_ECON_1_SUBGOAL_MST_IDS = []
RP_ECON_2_SUBGOAL_MST_IDS = []

RP_ECON_3_1_5_PERC_UNEMPLOYMENT_SUBG_MST_ID = 1 # 1. 5% unemployment
RP_ECON_3_2_USER_SPEC_RATE_SUBG_MST_ID      = 2 # 2. User-specified rate

RP_ECON_3_SUBGOAL_MST_IDS = [
    RP_ECON_3_1_5_PERC_UNEMPLOYMENT_SUBG_MST_ID,
    RP_ECON_3_2_USER_SPEC_RATE_SUBG_MST_ID,
]

RP_ECON_4_1_ELIM_NEET_SUBG_MST_ID               = 1 # 3. Eliminate NEET
RP_ECON_4_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID = 2 # 1. Achieve gender equity
RP_ECON_4_3_USER_SPEC_PERC_SUBG_MST_ID          = 3 # 4. User-specified %

RP_ECON_4_SUBGOAL_MST_IDS = [
    RP_ECON_4_1_ELIM_NEET_SUBG_MST_ID,
    RP_ECON_4_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID,
    RP_ECON_4_3_USER_SPEC_PERC_SUBG_MST_ID,
]

RP_ECON_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
    RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID  : RP_ECON_3_1_5_PERC_UNEMPLOYMENT_SUBG_MST_ID,
    RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID  : RP_ECON_4_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID,
}

#########################################   Build Your Own   ##########################################

RP_BYO_1_SUBGOAL_MST_IDS = []

# BYO has no default subgoals for its one goal
RP_BYO_SECTOR_DEFAULT_SUBGOAL_MST_IDS = {
}

#######################################################################################################

RP_SECTOR_GOALS = {
    RP_EDUCATION_SECTOR_MST_ID      : RP_EDU_GOAL_MST_IDS,
    RP_HEALTH_SECTOR_MST_ID         : RP_HLTH_GOAL_MST_IDS,
    RP_WASH_SECTOR_MST_ID           : RP_WASH_GOAL_MST_IDS,
    RP_FOOD_SECURITY_SECTOR_MST_ID  : RP_FOOD_GOAL_MST_IDS,
    RP_URBANIZATION_SECTOR_MST_ID   : RP_URB_GOAL_MST_IDS,
    RP_ENERGY_SECTOR_MST_ID         : RP_ENERGY_GOAL_MST_IDS,
    RP_ECONOMIC_SECTOR_MST_ID       : RP_ECON_GOAL_MST_IDS,
    RP_BUILD_YOUR_OWN_SECTOR_MST_ID : RP_BYO_GOAL_MST_IDS,
}

RP_SECTOR_DEFAULT_SUBGOALS = {
    RP_EDUCATION_SECTOR_MST_ID      : RP_EDU_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_HEALTH_SECTOR_MST_ID         : RP_HLTH_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_WASH_SECTOR_MST_ID           : RP_WASH_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_FOOD_SECURITY_SECTOR_MST_ID  : RP_FOOD_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_URBANIZATION_SECTOR_MST_ID   : RP_URB_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_ENERGY_SECTOR_MST_ID         : RP_ENERGY_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_ECONOMIC_SECTOR_MST_ID       : RP_ECON_SECTOR_DEFAULT_SUBGOAL_MST_IDS,
    RP_BUILD_YOUR_OWN_SECTOR_MST_ID : RP_BYO_SECTOR_DEFAULT_SUBGOAL_MST_IDS
}

RP_GOAL_SUBGOALS = {

    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID}"                    : RP_EDU_1_SUBGOAL_MST_IDS, 
    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID}"                     : RP_EDU_2_SUBGOAL_MST_IDS,
    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID}"          : RP_EDU_3_SUBGOAL_MST_IDS,

    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID}"              : RP_HLTH_1_SUBGOAL_MST_IDS,
    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID}"       : RP_HLTH_2_SUBGOAL_MST_IDS,
    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID}"               : RP_HLTH_3_SUBGOAL_MST_IDS,

    f"{RP_WASH_SECTOR_MST_ID}_{RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID}"              : RP_WASH_1_SUBGOAL_MST_IDS, 
    f"{RP_WASH_SECTOR_MST_ID}_{RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID}"                   : RP_WASH_2_SUBGOAL_MST_IDS, 

    f"{RP_FOOD_SECURITY_SECTOR_MST_ID}_{RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID}" : RP_FOOD_1_SUBGOAL_MST_IDS, 
    f"{RP_FOOD_SECURITY_SECTOR_MST_ID}_{RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID}"              : RP_FOOD_2_SUBGOAL_MST_IDS, 

    f"{RP_URBANIZATION_SECTOR_MST_ID}_{RP_URB_1_SLOW_INCR_NUM_PEOPLE_GOAL_MST_ID}"             : RP_URB_1_SUBGOAL_MST_IDS, 
    f"{RP_URBANIZATION_SECTOR_MST_ID}_{RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID}"           : RP_URB_2_SUBGOAL_MST_IDS, 

    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID}"                   : RP_ENERGY_1_SUBGOAL_MST_IDS, 
    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID}"          : RP_ENERGY_2_SUBGOAL_MST_IDS, 
    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID}"              : RP_ENERGY_3_SUBGOAL_MST_IDS, 

    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_1_REDUCE_AGE_DEPEND_RATIO_GOAL_MST_ID}"             : RP_ECON_1_SUBGOAL_MST_IDS, 
    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_2_INCR_GDP_PER_CAPITA_GOAL_MST_ID}"                 : RP_ECON_2_SUBGOAL_MST_IDS, 
    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID}"                   : RP_ECON_3_SUBGOAL_MST_IDS, 
    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID}"                   : RP_ECON_4_SUBGOAL_MST_IDS, 

    # f"{RP_BUILD_YOUR_OWN_SECTOR_MST_ID}_{RP_BYO_1_BYO_GOAL_MST_ID}"                            : RP_BYO_1_SUBGOAL_MST_IDS, 
}

RP_GOAL_USER_SPEC_SUBGOALS = {

    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID}"                    : RP_EDU_1_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID, 
    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID}"                     : RP_EDU_2_3_USER_SPEC_PERC_CHILDREN_SUBG_MST_ID,
    f"{RP_EDUCATION_SECTOR_MST_ID}_{RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID}"          : RP_EDU_3_2_USER_SPEC_RATIO_SUBG_MST_ID,

    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID}"              : RP_HLTH_1_3_USER_SPEC_PERC_BIRTHS_SUBG_MST_ID,
    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID}"       : RP_HLTH_2_3_USER_SPEC_PERC_PREGNANCIES_SUBG_MST_ID,
    f"{RP_HEALTH_SECTOR_MST_ID}_{RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID}"               : RP_HLTH_3_2_USER_SPEC_NUM_SUBG_MST_ID,

    f"{RP_WASH_SECTOR_MST_ID}_{RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID}"              : RP_WASH_1_2_USER_SPEC_PERC_HH_SUBG_MST_ID, 
    f"{RP_WASH_SECTOR_MST_ID}_{RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID}"                   : RP_WASH_2_2_USER_SPEC_PERC_HH_SUBG_MST_ID, 

    f"{RP_FOOD_SECURITY_SECTOR_MST_ID}_{RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID}" : RP_FOOD_1_2_USER_SPEC_PERC_DOM_GROWN_SUBG_MST_ID, 
    f"{RP_FOOD_SECURITY_SECTOR_MST_ID}_{RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID}"              : RP_FOOD_2_3_USER_SPEC_PERC_FOOD_INSECURE_SUBG_MST_ID, 

    # NA f"{RP_URBANIZATION_SECTOR_MST_ID}_{RP_URB_1_SLOW_INCR_NUM_PEOPLE_GOAL_MST_ID}"             : 0, 
    f"{RP_URBANIZATION_SECTOR_MST_ID}_{RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID}"           : RP_URB_2_3_USER_SPEC_PERC_URBAN_SUBG_MST_ID, 

    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID}"                   : RP_ENERGY_1_2_USER_SPEC_PERC_POP_SUBG_MST_ID, 
    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID}"          : RP_ENERGY_2_2_USER_SPEC_PERC_POP_SUBG_MST_ID, 
    f"{RP_ENERGY_SECTOR_MST_ID}_{RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID}"              : RP_ENERGY_3_2_USER_SPEC_PERC_POP_SUBG_MST_ID, 

    #f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_1_REDUCE_AGE_DEPEND_RATIO_GOAL_MST_ID}"             : 0, 
    #f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_2_INCR_GDP_PER_CAPITA_GOAL_MST_ID}"                 : 0, 
    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID}"                   : RP_ECON_3_2_USER_SPEC_RATE_SUBG_MST_ID, 
    f"{RP_ECONOMIC_SECTOR_MST_ID}_{RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID}"                   : RP_ECON_4_3_USER_SPEC_PERC_SUBG_MST_ID, 

    #f"{RP_BUILD_YOUR_OWN_SECTOR_MST_ID}_{RP_BYO_1_BYO_GOAL_MST_ID}"                            : 0, 
}

#######################################################################################################

# FP Scenario Goal options

RP_FP_SCEN_GOAL_ACCELERATE_MST_ID = 1
RP_FP_SCEN_GOAL_MEDIAN_MST_ID = 2
RP_FP_SCEN_GOAL_CUSTOM_MST_ID = 3

# FP Scenario Counterfactual (CF) options

RP_FP_SCEN_CF_MEDIAN_MST_ID   = 1
RP_FP_SCEN_CF_FLATLINE_MST_ID = 2
RP_FP_SCEN_CF_CUSTOM_MST_ID   = 3

# FP Scenarios

RP_FP_SCEN_STATUS_QUO_MST_ID     = 1
RP_FP_SCEN_GOAL_MST_ID           = 2 # 'Accelerated' in spreadsheet sometimes for some reason
RP_FP_SCEN_COUNTERFACTUAL_MST_ID = 3
RP_NUM_FP_SCENARIOS              = 3

RP_FP_SCEN_LIST = [RP_FP_SCEN_STATUS_QUO_MST_ID, RP_FP_SCEN_GOAL_MST_ID, RP_FP_SCEN_COUNTERFACTUAL_MST_ID]
RP_FP_SCEN_NO_STATUS_QUO_LIST = [RP_FP_SCEN_GOAL_MST_ID, RP_FP_SCEN_COUNTERFACTUAL_MST_ID]
RP_FP_SCEN_NO_COUNTERFACTUAL_LIST = [RP_FP_SCEN_STATUS_QUO_MST_ID, RP_FP_SCEN_GOAL_MST_ID]

# FP Scenario Scale-up (SC) Options

RP_FP_SCEN_SC_ACCELERATED = 1
RP_FP_SCEN_SC_MEDIAN      = 2
RP_FP_SCEN_SC_FLATLINE    = 3
RP_FP_SCEN_NUM_SC_OPTIONS = 3

# Target populations for BYO sector

RP_TP_TOTAL_POP_MST_ID        = 1
RP_TP_WOMEN_REPROD_AGE_MST_ID = 2
RP_TP_URBAN_POP_MST_ID        = 3
RP_TP_YOUTH_15_TO_29_MST_ID   = 4
RP_TP_HOUSEHOLDS_MST_ID       = 5
RP_TP_CHILDREN_UNDER_2_MST_ID = 6

# SSP scenarios

RP_SSP_BASE_MST_ID = 1
RP_SSP1_MST_ID     = 2
RP_SSP2_MST_ID     = 3
RP_SSP3_MST_ID     = 4
RP_SSP5_MST_ID     = 5

RP_SSP_LIST        = [RP_SSP1_MST_ID, RP_SSP2_MST_ID, RP_SSP5_MST_ID]
RP_SSP_BASE_LIST   = [RP_SSP_BASE_MST_ID, RP_SSP1_MST_ID, RP_SSP2_MST_ID, RP_SSP5_MST_ID]
RP_SSP_BASE_3_LIST = [RP_SSP_BASE_MST_ID, RP_SSP1_MST_ID, RP_SSP2_MST_ID, RP_SSP3_MST_ID, RP_SSP5_MST_ID]




IH_IVG_MaternalNewbornHealth_MST_ID = '1'
IH_IVG_SexualReprodHealth_MST_ID = '111'
IH_IVG_ChildHealth_MST_ID = '15'
IH_IVG_Vaccination_MST_ID = '20'
IH_IVG_Malaria_MST_ID = '21'
IH_IVG_TB_MST_ID = '22'
IH_IVG_HIVAIDS_MST_ID = '23'
IH_IVG_Nutrition_MST_ID = '28'
IH_IVG_WASH_MST_ID = '29'
IH_IVG_NoncommunicableDiseases_MST_ID = '30'
IH_IVG_MentalNueroSubsUseDis_MST_ID = '42'
IH_IVG_AdolescentHealth_MST_ID = '73'
IH_IVG_NegTropDiseases_MST_ID = '74'
IH_PROG_AREA_MST_IDS = [IH_IVG_MaternalNewbornHealth_MST_ID, IH_IVG_SexualReprodHealth_MST_ID, IH_IVG_ChildHealth_MST_ID, IH_IVG_Vaccination_MST_ID, IH_IVG_Malaria_MST_ID, IH_IVG_TB_MST_ID, IH_IVG_HIVAIDS_MST_ID, IH_IVG_Nutrition_MST_ID, IH_IVG_WASH_MST_ID, IH_IVG_NoncommunicableDiseases_MST_ID, IH_IVG_MentalNueroSubsUseDis_MST_ID, IH_IVG_AdolescentHealth_MST_ID, IH_IVG_NegTropDiseases_MST_ID]



import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.PJ.PJNTags as pjmvt

# import SpectrumCommon.Const.PC.PCConstLink as pccl
# import SpectrumCommon.Const.PC.PCTags as pcmvt
# import SpectrumCommon.Modvars.PC.PCModVarDefs as pcmvd

# import SpectrumCommon.Modvars.PC.PCReadDatabases as pcrd

def init_modvars(countryID, first_year, final_year, extra):
    '''
        Initializes HIS MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''
    
    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False
    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }

    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)

    # Load databases
    # cost_cat_DB = pcrd.PC_load_cost_cat_DB()
    # cost_input_DB = pcrd.PC_load_cost_input_DB()
    # costing_sect_DB = pcrd.PC_load_costing_sect_DB()
    # activity_DB = pcrd.PC_load_activity_DB()
    # activity_line_item_DB = pcrd.PC_load_activity_line_item_DB()

    # value_bool = False
    # value_int = 0
    # value_flt = 0.0
    # value_str = ''
    # value_date = date.today().strftime("%B %d, %Y"),   

    num_years_inputs = final_year - first_year + 1

    #prog_area_ID_list = pcrd.get_prog_area_ID_list(group_DB)
    #area_data_list = pcmvd.init_area_dict_list(prog_area_ID_list, num_total_years)
    # area_data_list = pcmvd.PC_init_area_dict_list(pccl.PC_IH_PROG_AREA_MST_ID, extra[pccl.PC_IH_PROG_AREA_RECORDS_TAG_V1], num_years_inputs, costing_mode)
    
    # cost_cat_records = pcrd.PC_init_cost_cat_records(cost_cat_DB)
    # cost_input_records = pcrd.PC_init_cost_input_records(cost_input_DB)
    # costing_sect_records = pcrd.PC_init_costing_sect_records(costing_sect_DB, num_years_inputs, costing_mode)
    # activity_records = pcrd.PC_init_activity_records(activity_DB, costing_mode)
    # activity_line_item_records = pcrd.PC_init_activity_line_item_records(activity_line_item_DB, num_years_inputs, costing_mode)

    # deliv_chan_pack_record_list []
    # nat_prog_record_list = []
    # audit_record_list = []
    # locked_area_record_list = []

    # add flag modvars to one modes modvar. If flag is in set, turn mode on. Else, turn mode off. Just like how you will do with intervention sets in IC.

    mod_vars_rec = {            
        # pcmvt.PC_DATA_CHANGED_TAG_V1          : value_bool, 
        # pcmvt.PC_PROJ_VALID_TAG_V1            : value_bool, 
        # pcmvt.PC_FILE_NAME_TAG_V1             : value_str,                          
        # pcmvt.PC_PROJ_VALID_DATE_TAG_V1       : value_date,    

        # pcmvt.PC_AREA_DATA_TAG_V1             : area_data_list,
        
        # pcmvt.PC_COST_CAT_RECORDS_TAG_V1      : cost_cat_records,
        # pcmvt.PC_COSTING_SECT_RECORDS_TAG_V1  : costing_sect_records, 
        # pcmvt.PC_COST_INPUT_RECORDS_TAG_V1    : cost_input_records,
        # pcmvt.PC_ACT_RECORDS_TAG_V1           : activity_records, 
        # pcmvt.PC_ACT_LINE_ITEM_RECORDS_TAG_V1 : activity_line_item_records,

        # pcmvt.PC_RES_COSTS_BY_SECT_TAG_V1    : [],
        # pcmvt.PC_RES_COSTS_BY_SUBSECT_TAG_V1 : [],
        # pcmvt.PC_RES_COSTS_BY_ACT_TAG_V1     : [],
        # pcmvt.PC_RES_COSTS_BY_CAT_TAG_V1     : [],

        # pcmvt.PC_STD_ROW_INFO_META_TAG_V1 : pcmvd.PC_init_row_info_meta(gbc.GB_NOT_FOUND), # Standard
        # pcmvt.PC_HIV_ROW_INFO_META_TAG_V1 : pcmvd.PC_init_row_info_meta(pccl.PC_IH_HIV_AIDS_MST_ID) # HIV
    }

    # if costing_mode == gbc.GB_LIST_COSTING_MODE:
    #     mod_vars_rec[pcmvt.PC_ENTER_COSTING_SECT_COSTS_ALL_YEARS_CS_TAG_V1] = False

    #Debugging
    #with open('C:\Proj\IHTSampleFile.JSON', 'w') as f:
    #   json.dump(mod_vars_dict, f)

    return mod_vars_rec

    


import numpy as np
import scipy
import scipy.special
import scipy.stats




# Function to find log-density functions matching given values for the 2.5, 50 and 97.5th percentiles
# 'prctiles' is a vector of the [2.5, 50, 97.5]th percentiles
# 'distribution' specifies whether log normal or beta 
# 'visualising' should be set to 'true' if wanting to see the estimated pdf

def logncdf(x,y,z):
    return 0

def betacdf(x,y,z):
    
    return 0

    
def get_distribution_fns(prctiles, distribution, visualising):
    aux = {}
    val=1
    dat  = np.sort(prctiles)
    opts = []#optimset('TolX',1e-12,'TolFun',1e-12,'MaxIter',1e6,'MaxFunEvals',1e6)

    # Estimators to help setting up initial guesses
    mn = dat[1] 
    var = (dat[2]-dat[0])**(2/4)

    if distribution == 'lognorm':
        
        ff  = lambda x : [scipy.stats.lognorm.cdf(dat[0],x[0],x[1]), scipy.stats.lognorm.cdf(dat[1],x[0],x[1]), scipy.stats.lognorm.cdf(dat[2],x[0],x[1])]

        # Set up initial guess
        sig = np.sqrt(np.log(var/(mn**2)+1))
        mu  = np.log(mn)-(sig**2)/2 
        init = [mu, sig]

        # Do the optimisation
        perc = np.array([2.5, 50, 97.5])
        obj = lambda x : sum((ff(x)/(perc/100) - 1)**2)
        out = scipy.optimize.fmin(obj, init)# fminsearch(obj, init, opts)
        #if val > 1e-2:
        #    raise  Exception('Calibration setup not converged')
        
        # Get the log-pdf
        mu = out[0]
        sig = out[1]
        logfn = lambda x : -(np.log(x)-mu)**2/(2*sig**2) - np.log(x*sig*np.sqrt(2*np.pi))
        
    elif distribution == 'beta':
        ff  = lambda x : [scipy.stats.beta(dat[0],x[0],x[1]), scipy.stats.beta(dat[1],x[0],x[1]), scipy.stats.beta(dat[2],x[0],x[1])]

        # Set up initial guess
        tmp = mn*(1-mn)/var-1
        a = tmp*mn 
        b = tmp*(1-mn) 
        init = [a, b]

        # Do the optimisation
        obj = lambda x: sum((ff(x)/([2.5, 50, 97.5]/100) - 1)**2)
        [out, val] = scipy.optimize.fmin(obj, init)#fminsearch(obj,init,opts);
        if (val > 1e-2):
            raise Exception('Calibration setup not converged')
        
        # Get the log-pdf
        a = out[0] 
        b = out[1]
        logfn = lambda x : (a-1)*np.log(x) + (b-1)*np.log(1-x) - scipy.special.betaln(a,b)
    

    aux['sim'] = ff(out)                                                         # Simulated values of CDF at given percentile points
    aux['val'] = val                                                             # The final objective function

    '''
    if nargin == 3 && visualising
        figure;
        x = linspace(dat(1)*.8, dat(3)*1.2);
        if strcmp(distribution,'lognorm')
            y = lognpdf(x,out(1),out(2));
        elseif strcmp(distribution,'beta')
            y = betapdf(x,out(1),out(2));
      
        plot(x,y); hold on;
        # Show the percentiles
        for id = 1:3
            line(dat(id)*[1 1],ylim,'linestyle','--');
    '''
      

    return logfn, out, aux





import numpy as np

from AvenirCommon.Util import *
from SpectrumCommon.Const.FP import *
from SpectrumCommon.Modvars.FP.FPUtil import * 
from SpectrumCommon.Const.GB  import *

def normalize(L, total):
    sum = 0.0
    norm = [0.0] * len(L)

    for i in range(0, (len(L))):
        sum = sum + L[i]

    if sum == 0.0 and np.isclose(sum, 0.0,  abs_tol = 0.00001):
        multiplier = 0.0
    else:
        multiplier = total / sum

    for i in range(0, (len(L))):
        norm[i] = L[i] * multiplier
    
    return norm

def gbNormalize(L):
    valueGreatThanOne = False
    result = []
    for i in range(0, (len(L)-1)):
        if L[i] > 1.0: #and isclose(sum, 1.0,  abs_tol = 0.00001):
            valueGreatThanOne = True
    
    if valueGreatThanOne:
        result = normalize(L, 100)
    else:
        result = normalize(L, 1)
    return result


def ActiveMethod_Init (db):
    result = [False] * (FP_Max_Methods+1)
    #for m = 0 to FP_Max_Methods do
    #   result[m] = False
   # result[FP_CondomML] = True
   # result[FP_CondomFM] = True

    result[FP_CondomML] = np.any(db['condomMale'])      
    result[FP_fster] =     np.any(db['sterilizationFemale'])       
    result[FP_mster] =  np.any(db['sterilizationMale'])         
    result[FP_vag_barrier] = np.any(db['vaginalBarrierDiaphragm'])   
    result[FP_withdrawal] =   np.any(db['traditionalWithdrawal'])   
    result[FP_PerAbst]= np.any(db['traditionalPeriodicAbstinence'])   
    result[FP_Traditional] =  np.any(db['traditionalOther'])   
    result[FP_inj3month]=   np.any(db['injectableDepo3MoDefault'])   
    result[FP_Jadelle] =  np.any(db['implantJadelle5YrDefault'])      
    result[FP_StandardPill] = np.any(db['pillStandard'])                       
    result[FP_IUDCopper] = np.any(db['IUD_CopperT10Yr'])                
    result[FP_other] =      np.any(db['otherModern'])      

    result = [bool(val) for val in result]         

    return result

def Sources_Init ():
    result = [''] * (FP_max_sources+1)
    result[0] = 'All'
    result[1] = 'Default Source'
    return result

def MethodName_Init():
    result = []
    for m in GBRange(0, FP_Max_Methods):
        result += [GetFP_methodName(m)]
    return result

def MethodMix_Init(db, final_year_idx):
    result = np.zeros((FP_MAX_AGE + 1, FP_Max_Methods + 1, FP_max_need + 1, final_year_idx+1))

    need = FP_all_need

    result[:,FP_CondomML,need] = db['condomMale']              
    result[:,FP_fster,need] = db['sterilizationFemale']             
    result[:,FP_mster,need] = db['sterilizationMale']             
    result[:,FP_vag_barrier,need] = db['vaginalBarrierDiaphragm'] 
    result[:,FP_withdrawal,need] = db['traditionalWithdrawal']    
    result[:,FP_PerAbst,need]= db['traditionalPeriodicAbstinence'] 
    result[:,FP_Traditional,need] = db['traditionalOther']        
    result[:,FP_inj3month,need]= db['injectableDepo3MoDefault']         
    result[:,FP_Jadelle,need] = db['implantJadelle5YrDefault']         
    result[:,FP_StandardPill,need] = db['pillStandard']                       
    result[:,FP_IUDCopper,need] = db['IUD_CopperT10Yr']               
    result[:,FP_other,need] = db['otherModern']                   

    return result


def Effectiveness_Init(eff_db, final_year_idx):
    result = np.zeros((FP_MAX_AGE + 1, FP_Max_Methods + 1, final_year_idx+1))

    for eff in (eff_db):
        #reshape effectiveness values to go by age and year
        eff_years = np.swapaxes(np.array([eff['values']]*(final_year_idx+1)), 0, 1)
        result[:, eff['mstID'], :] += eff_years
                          
    return result

def UnitsPerCYP_Init(final_year_idx):
    result = np.zeros((FP_Max_Methods + 1, final_year_idx+1))
    result[FP_all_methods]  += 105.0
    result[FP_CondomFM]     += 120.0
    result[FP_CondomML]     += 120.0
    result[FP_inj3month]    += 4.0
    result[FP_inj2month]    += 6.0
    result[FP_inj1month]    += 13.0
    result[FP_Uninject]     += 4.0
    result[FP_Implanon]     += 2.5
    result[FP_Jadelle]      += 3.8
    result[FP_Sino_Implant] += 3.2
    result[FP_StandardPill] += 15.0
    result[FP_Progestin]    += 15.0
    result[FP_Peri_coital]  += 15.0
    result[FP_VFT]          += 120.0
    result[FP_LAM]          += 0.25
    result[FP_SDM]          += 1.5

    return result
                
def AverageAge_Init(countryID, final_year_idx):
    result = np.zeros((FP_Max_Methods+1, final_year_idx+1))

    if countryID in ['IND', 'NPL', 'BGD']: #india, nepal, bangladesh
        result[FP_mster] += 26.0 
        result[FP_fster] += 26.0 
    else:
        result[FP_mster] += 35.0 
        result[FP_fster] += 35.0 

    return result    
      
def DurationOfUse_Init(final_year_idx):
    result = np.zeros((FP_Max_Methods+1, final_year_idx+1))

    result[FP_IUDCopper]    += 4.6
    result[FP_IUDLngIus]    += 3.3
    result[FP_Implanon]     += 2.5
    result[FP_Jadelle]      += 3.8
    result[FP_Sino_Implant] += 3.2
    result[FP_vag_barrier]  += 1.0

    return result


def prevalence_Init(cpr, final_year_idx):
    result = np.zeros((FP_MAX_AGE+1, FP_max_need+1, final_year_idx+1))
    result[:, FP_all_need] = cpr
    return result
 
 
def unmetNeed_Init(unmet_need, final_year_idx):
    result = np.zeros((FP_MAX_AGE+1, FP_max_need+1, final_year_idx+1))
    result[:, FP_all_need] = unmet_need
    return result 
    
   

def ChildSurvInputs_Init(db):
    result = np.zeros((FP_RelU5MR+1))
    result[FP_RelRiskyBirth] = -0.55
    result[FP_RelIMR]        = 2.31
    result[FP_RelU5MR]       = 3.54
    result[FP_PerBirthRisk]  = 0#db['anyFertilityRisk']
    return result


def PACInputs_Init(MMR, final_year_idx):
    result = np.zeros((FP_CostCounServCase+1, final_year_idx+1))

    # # PAC Default values 
    result[FP_MortalityUntreatVsTreat] += 3.0
    result[FP_MMRMaternDeath][0] = MMR 
    return result

def LAMinsusceptible_Init():
    result = np.zeros((FP_LAM4_5+1))
    result[FP_LAM0_1] = 0.96
    result[FP_LAM2_3] = 0.906
    result[FP_LAM4_5] = 0.815
    return result

    

def defaultRelativeRisk(db):
    risks = [db['MA_L18FirstBirth'],
             db['MA_L18_BO2_3'],
             db['MA_L18_BO_G3'],
             db['MA18_34BO2_3'],
             db['MA18_34BO_G3'],
             db['MA_G35BO2_3'],
             db['MA_G35BO_G3'],
             db['MA18_34FirstBirth'],
             db['MA_G35FirstBirth']]
             
    #Database values should be already normalized
    #gbNormalize(risks)
    total = sum(risks)
    result =  [r/total*100 for r in risks]
    return result

def defaultBirthInterval(db):
    birth_interval = [db['firstBirthInterval'],
                      db['L18Months'],
                      db['BO18_24Months'],
                      db['BO_24MonthsOrGreater']]

    #Database values should be already normalized
    #gbNormalize(birth_interval)#maybe normalize risk in blob storage
    total = sum(birth_interval)
    result =  [r/total*100 for r in birth_interval]
    return result

def defaultRelRiskCoef(ageBirthAdv, birthIntAdv, birthOrderAdv):
    #set advance constants, ask john of bob for values
      #First Birth
    ageBirthAdv[FP_RRIntrcept][FP_RRFirstBirth][FP_RRLess18] =  7.265480
    ageBirthAdv[FP_RRC1][FP_RRFirstBirth][FP_RRLess18]       = -0.091041
    ageBirthAdv[FP_RRC2][FP_RRFirstBirth][FP_RRLess18]       =  0.001016

    ageBirthAdv[FP_RRIntrcept][FP_RRFirstBirth][FP_RR18To34] =  8.798430
    ageBirthAdv[FP_RRC1][FP_RRFirstBirth][FP_RR18To34]       =  0.303922
    ageBirthAdv[FP_RRC2][FP_RRFirstBirth][FP_RR18To34]       = -0.000318

    ageBirthAdv[FP_RRIntrcept][FP_RRFirstBirth][FP_RRMore34] =  0.125492
    ageBirthAdv[FP_RRC1][FP_RRFirstBirth][FP_RRMore34]       = -0.007625
    ageBirthAdv[FP_RRC2][FP_RRFirstBirth][FP_RRMore34]       =  0.000222

      #Second to third birth
    ageBirthAdv[FP_RRIntrcept][FP_RRSecToThirdBirth][FP_RRLess18] =  2.161788
    ageBirthAdv[FP_RRC1][FP_RRSecToThirdBirth][FP_RRLess18]       = -0.047509
    ageBirthAdv[FP_RRC2][FP_RRSecToThirdBirth][FP_RRLess18]       =  0.000438

    ageBirthAdv[FP_RRIntrcept][FP_RRSecToThirdBirth][FP_RR18To34] = 26.088305
    ageBirthAdv[FP_RRC1][FP_RRSecToThirdBirth][FP_RR18To34]       =  0.274474
    ageBirthAdv[FP_RRC2][FP_RRSecToThirdBirth][FP_RR18To34]       = -0.001101

    ageBirthAdv[FP_RRIntrcept][FP_RRSecToThirdBirth][FP_RRMore34] =  0.696853
    ageBirthAdv[FP_RRC1][FP_RRSecToThirdBirth][FP_RRMore34]       = -0.022429
    ageBirthAdv[FP_RRC2][FP_RRSecToThirdBirth][FP_RRMore34]       =  0.000796

      #Four or more births
    ageBirthAdv[FP_RRIntrcept][FP_RRMoreFourBirth][FP_RRLess18] =  0.047730
    ageBirthAdv[FP_RRC1][FP_RRMoreFourBirth][FP_RRLess18]       = -0.001463
    ageBirthAdv[FP_RRC2][FP_RRMoreFourBirth][FP_RRLess18]       =  0.000012

    ageBirthAdv[FP_RRIntrcept][FP_RRMoreFourBirth][FP_RR18To34] = 39.282154
    ageBirthAdv[FP_RRC1][FP_RRMoreFourBirth][FP_RR18To34]       = -0.344732
    ageBirthAdv[FP_RRC2][FP_RRMoreFourBirth][FP_RR18To34]       = -0.000165

    ageBirthAdv[FP_RRIntrcept][FP_RRMoreFourBirth][FP_RRMore34] = 15.168927
    ageBirthAdv[FP_RRC1][FP_RRMoreFourBirth][FP_RRMore34]       = -0.061253
    ageBirthAdv[FP_RRC2][FP_RRMoreFourBirth][FP_RRMore34]       = -0.000866


    birthOrderAdv[FP_RRIntrcept][FP_RRBOGreater3] = -18.90215906
    birthOrderAdv[FP_RRC1][FP_RRBOGreater3]       =  19.41538934
    birthOrderAdv[FP_RRC2][FP_RRBOGreater3]       = -1.32560387

    birthOrderAdv[FP_RRIntrcept][FP_RRMAGreat35] = -1.940569547
    birthOrderAdv[FP_RRC1][FP_RRMAGreat35]       =  5.700909531
    birthOrderAdv[FP_RRC2][FP_RRMAGreat35]       = -0.471392363


    birthIntAdv[FP_RRIntrcept][FP_RRFirstBirth] = 16.554243
    birthIntAdv[FP_RRC1][FP_RRFirstBirth]       =  0.202914
    birthIntAdv[FP_RRC2][FP_RRFirstBirth]       =  0.000887

    birthIntAdv[FP_RRIntrcept][FP_RRLess18Months] = 1.854915
    birthIntAdv[FP_RRC1][FP_RRLess18Months]       = -0.011982
    birthIntAdv[FP_RRC2][FP_RRLess18Months]       = -0.000019

    birthIntAdv[FP_RRIntrcept][FP_RR18to23Months] = 5.648637
    birthIntAdv[FP_RRC1][FP_RR18to23Months]       = 0.029261
    birthIntAdv[FP_RRC2][FP_RR18to23Months]       = -0.000311

    birthIntAdv[FP_RRIntrcept][FP_RRMore24Months] = 75.942205
    birthIntAdv[FP_RRC1][FP_RRMore24Months]       = -0.220192
    birthIntAdv[FP_RRC2][FP_RRMore24Months]       = -0.000557


def relativeRisks_Init(db, final_year_idx):
    #result = [[[0]*(final_year_idx+1)]*FP_RRAgeOrderMax]*FP_RRBirthOrderMax

    ageBirthOrder = np.zeros((FP_RRBirthOrderMax + 1, FP_RRAgeOrderMax + 1))
    birthInterval = np.zeros(FP_RRBirthIntervalMax + 1)
    ageBirthAdv   = np.zeros((FP_RRC2 + 1, FP_RRBirthOrderMax + 1, FP_RRAgeOrderMax + 1))
    birthIntAdv   = np.zeros((FP_RRC2 + 1, FP_RRExtraBirthIntervals + 1))
    birthOrderAdv = np.zeros(( FP_RRC2 + 1, FP_RRMA18To34 + 1))
   

    default_risks = defaultRelativeRisk(db)
    ageBirthOrder[FP_RRFirstBirth][FP_RRLess18]       = default_risks[0] 
    ageBirthOrder[FP_RRSecToThirdBirth][FP_RRLess18]  = default_risks[1] 
    ageBirthOrder[FP_RRMoreFourBirth][FP_RRLess18]    = default_risks[2] 
    ageBirthOrder[FP_RRSecToThirdBirth][FP_RR18To34]  = default_risks[3] 
    ageBirthOrder[FP_RRMoreFourBirth][FP_RR18To34]    = default_risks[4] 
    ageBirthOrder[FP_RRSecToThirdBirth][FP_RRMore34]  = default_risks[5] 
    ageBirthOrder[FP_RRMoreFourBirth][FP_RRMore34]    = default_risks[6]
    ageBirthOrder[FP_RRFirstBirth][FP_RR18To34]       = default_risks[7]
    ageBirthOrder[FP_RRFirstBirth][FP_RRMore34]       = default_risks[8]

    default_birth_interval = defaultBirthInterval(db)
    birthInterval[FP_RRFirstBirth]    = default_birth_interval[0] 
    birthInterval[FP_RRLess18Months]  = default_birth_interval[1]
    birthInterval[FP_RR18to23Months]  = default_birth_interval[2] 
    birthInterval[FP_RRMore24Months]  = default_birth_interval[3]

    defaultRelRiskCoef(ageBirthAdv, birthIntAdv, birthOrderAdv)

    # # flatline result values with baseline values
    # for t in GBRange(1, final_year_idx):
    #     for i in GBRange(1, FP_RRBirthOrderMax):
    #         for ma in GBRange(1, FP_RRAgeOrderMax):
    #             ageBirthOrderResult[i][ma][t] = ageBirthOrder[i][ma]

    #     for i in GBRange(1, FP_RRBirthIntervalMax):
    #         birthIntervalResult[i][t] = birthInterval[i]
    return {
        'ageBirthOrder' : ageBirthOrder.tolist(),      
        'birthInterval' : birthInterval.tolist(), 
        
        'ageBirthAdv'   : ageBirthAdv.tolist(), 
        'birthIntAdv'   : birthIntAdv.tolist(), 
        'birthOrderAdv' : birthOrderAdv.tolist(),
        
        }


def FPSource_Init(db):
    group = db['Group']
    country_name = db['name']
    main_data_sources = db['mainDataSources']
    result = main_data_sources

    # if (group == 'A') or (group == 'AB'):
    #     result = f'Survey data are from: {main_data_sources} {country_name}'
    # elif (group == 'B'):
    #     result = f'Contraceptive prevalence is from: {main_data_sources} {country_name} \n Other proximate determinants data are based upon averages calculated for countries with similar levels of fertility.'
    # elif (group == 'C'):
    #     result = 'All contraceptive information and other proximate determinants data are based upon averages calculated for countries with similar levels of fertility.'
    # else:
    #     result = main_data_sources

    # result += '\n\nAbortions (i.e., terminations per unwanted pregnancy) is based on subregional estimations by Bearak et al 2020.'

    return {
        'main':result,
        'goal':result
    } 




IC_IV_ContraceptiveUse_MST_ID = '47'
IC_IV_FolicAcidSupp_MST_ID = '7'
IC_IV_SafeAbortServ_MST_ID = '73'
IC_IV_PostAbortCaseMgmt_MST_ID = '170'
IC_IV_EctopicPregCaseMgmt_MST_ID = '171'
IC_IV_BlanketIronSuppFort_MST_ID = '446'
IC_IV_PillStdDailyReg_MST_ID = '95'
IC_IV_PillProgestin_MST_ID = '380'
IC_IV_PillPCC_MST_ID = '381'
IC_IV_CondomM_MST_ID = '96'
IC_IV_CondomF_MST_ID = '373'
IC_IV_InjectableDepo_MST_ID = '97'
IC_IV_InjectableNor_MST_ID = '374'
IC_IV_InjectableLun_MST_ID = '375'
IC_IV_Injectable6mo_MST_ID = '376'
IC_IV_InjectableUniject_MST_ID = '377'
IC_IV_IUD_Copper_MST_ID = '98'
IC_IV_IUD_LNG_IUS_MST_ID = '382'
IC_IV_ImplantJad_MST_ID = '99'
IC_IV_ImplantImp_MST_ID = '378'
IC_IV_ImplantSin_MST_ID = '379'
IC_IV_FSterilization_MST_ID = '100'
IC_IV_MSterilization_MST_ID = '101'
IC_IV_LAM_MST_ID = '102'
IC_IV_SDM_MST_ID = '383'
IC_IV_VagBarrier_MST_ID = '103'
IC_IV_VagTabs_MST_ID = '104'
IC_IV_PeriAbstinence_MST_ID = '384'
IC_IV_Widthdrawal_MST_ID = '385'
IC_IV_TraditionalNotSpec_MST_ID = '386'
IC_IV_OtherContraceptives_MST_ID = '105'
IC_IV_TetToxVacc_MST_ID = '4'
IC_IV_IPT_Mal_MST_ID = '3'
IC_IV_SyphDetectTr_MST_ID = '1'
IC_IV_CalciumSupp_MST_ID = '72'
IC_IV_Omeg3FASupp_MST_ID = '533'
IC_IV_AspirinLowDose_MST_ID = '529'
IC_IV_Progesterone_MST_ID = '530'
IC_IV_IronSuppPreg_MST_ID = '235'
IC_IV_MultiMicronSuppPreg_MST_ID = '9'
IC_IV_BalEnergySupp_MST_ID = '8'
IC_IV_HyperDisCaseMgmt_MST_ID = '82'
IC_IV_DiabCaseMgmt_MST_ID = '87'
IC_IV_MalCaseMgmt_MST_ID = '172'
IC_IV_MgSO4MgmtPreEclampsia_MST_ID = '81'
IC_IV_FGRDetMgmt_MST_ID = '91'
IC_IV_PMTCT_MST_ID = '14'
IC_IV_ThermalCare_MST_ID = '168'
IC_IV_DelayedCordClamp_MST_ID = '535'
IC_IV_CleanCordCare_MST_ID = '255'
IC_IV_CleanBirthEnvironment_MST_ID = '11'
IC_IV_ImmDryAndAdditionalStim_MST_ID = '174'
IC_IV_PrerefMgmtLabor_MST_ID = '109'
IC_IV_TrPostpartHemor_MST_ID = '112'
IC_IV_NeoResusc_MST_ID = '15'
IC_IV_CortPretermLabor_MST_ID = '5'
IC_IV_AntipPRoM_MST_ID = '6'
IC_IV_MgSO4ForEclampsia_MST_ID = '83'
IC_IV_ParenteralAdminOfUter_MST_ID = '62'
IC_IV_ParenteralAdminOfAntibiotics_MST_ID = '505'
IC_IV_AssisVagDeliv_MST_ID = '506'
IC_IV_ManualRemovOfPlacent_MST_ID = '507'
IC_IV_RemovOfRetainProdOfConcep_MST_ID = '508'
IC_IV_SurgeryChildbirth_MST_ID = '509'
IC_IV_BloodTransfusion_MST_ID = '510'
IC_IV_InductionOfLabour_MST_ID = '94'
IC_IV_KangarooMothCare_MST_ID = '16'
IC_IV_FullSuppCarePremat_MST_ID = '322'
IC_IV_iKMC_MST_ID = '547'
IC_IV_ChangesInBF_MST_ID = '17'
IC_IV_CompFeedEOnly_MST_ID = '18'
IC_IV_CompFeedSE_MST_ID = '19'
IC_IV_NNVitASupp_MST_ID = '540'
IC_IV_VitASupp_MST_ID = '46'
IC_IV_ZincSupp_MST_ID = '24'
IC_IV_BasicSanitation_MST_ID = '42'
IC_IV_PointOfUseFilteredWater_MST_ID = '40'
IC_IV_PipedWater_MST_ID = '41'
IC_IV_HandWashWSoap_MST_ID = '43'
IC_IV_HygDisposalStools_MST_ID = '44'
IC_IV_ITN_IRS_MST_ID = '22'
IC_IV_BCG_Vacc_MST_ID = '49'
IC_IV_PolioVacc_MST_ID = '34'
IC_IV_PentaVacc_MST_ID = '175'
IC_IV_DPT_Vacc_MST_ID = '33'
IC_IV_HibVacc_MST_ID = '25'
IC_IV_HepBVacc_MST_ID = '167'
IC_IV_PneuVacc_MST_ID = '26'
IC_IV_RotavirusVacc_MST_ID = '45'
IC_IV_MeningA_MST_ID = '191'
IC_IV_MalVacc_MST_ID = '251'
IC_IV_MeasVacc_MST_ID = '21'
IC_IV_HPV_Vacc_MST_ID = '132'
IC_IV_CovidVacc_MST_ID = '679'
IC_IV_CholeraVacc_MST_ID = '680'
IC_IV_DengueVacc_MST_ID = '681'
IC_IV_EbolaVacc_MST_ID = '682'
IC_IV_HepAVacc_MST_ID = '683'
IC_IV_JapEncephVacc_MST_ID = '684'
IC_IV_RabiesVacc_MST_ID = '685'
IC_IV_SeasInfluenzaVacc_MST_ID = '686'
IC_IV_TetToxVaccImmunization_MST_ID = '687'
IC_IV_TickEncephVacc_MST_ID = '688'
IC_IV_TyphoidVacc_MST_ID = '689'
IC_IV_VaricellaVacc_MST_ID = '690'
IC_IV_YellowFeverVacc_MST_ID = '691'
IC_IV_CampaignCholeraVacc_MST_ID = '692'
IC_IV_CampaignMeaslesVacc_MST_ID = '693'
IC_IV_CampaignMeningitisVacc_MST_ID = '694'
IC_IV_CampaignPolioVacc_MST_ID = '695'
IC_IV_CampaignTetanusVacc_MST_ID = '696'
IC_IV_CampaignTyphoidVacc_MST_ID = '697'
IC_IV_CampaignYellowFeverVacc_MST_ID = '698'
IC_IV_MatSepsisCaseMgmt_MST_ID = '173'
IC_IV_OralAntibNeonSeps_MST_ID = '69'
IC_IV_InjectAntibNeonSeps_MST_ID = '70'
IC_IV_FullSuppCareNeonSepsPneu_MST_ID = '13'
IC_IV_ORS_MST_ID = '27'
IC_IV_AntibTrDysentery_MST_ID = '28'
IC_IV_ZincTrDiar_MST_ID = '29'
IC_IV_OralAntibPneu_MST_ID = '30'
IC_IV_OxAndPulseOxiForPneum_MST_ID = '528'
IC_IV_VitATrMeas_MST_ID = '31'
IC_IV_CaseMgmtMalTreat_MST_ID = '658'
IC_IV_ACTsMal_MST_ID = '32'
IC_IV_SAM_MST_ID = '71'
IC_IV_TrMAM_MST_ID = '237'
IC_IV_ART_MST_ID = '36'
IC_IV_PregWSleepITN_MST_ID = '135'
IC_IV_IndoorResidSpray_MST_ID = '136'
IC_IV_VectorCtrlOther_MST_ID = '319'
IC_IV_Larviciding_MST_ID = '320'
IC_IV_SeasMalChemoproph_MST_ID = '479'
IC_IV_MalDiagMalFevers_MST_ID = '318'
IC_IV_MalDiagNonMalFevers_MST_ID = '483'
IC_IV_MalTrCh5t14_MST_ID = '480'
IC_IV_MalTrAdExclPregW_MST_ID = '126'
IC_IV_TrSevMalFivePlus_MST_ID = '481'
IC_IV_TrNonMalParasPosFevers_MST_ID = '454'
IC_IV_WorkplaceVisit_MST_ID = '350'
IC_IV_CaseMgmtWorkDiseases_MST_ID = '351'
IC_IV_HealthAssessRetWork_MST_ID = '352'
IC_IV_IntervFocSexWorkersF_MST_ID = '147'
IC_IV_FSW_PrEP_MST_ID = '628'
IC_IV_IntervFocSexWorkersM_MST_ID = '253'
IC_IV_MSW_PrEP_MST_ID = '629'
IC_IV_IntervFocMSM_MST_ID = '148'
IC_IV_MSM_PrEP_MST_ID = '630'
IC_IV_TransgenderHIV_Prev_MST_ID = '631'
IC_IV_TransgenderPrEP_MST_ID = '632'
IC_IV_IDUOutreach_MST_ID = '149'
IC_IV_PWID_PrEP_MST_ID = '633'
IC_IV_IDUNeedleExchange_MST_ID = '196'
IC_IV_IDUDrugSubstit_MST_ID = '197'
IC_IV_PrisonersPrEP_MST_ID = '634'
IC_IV_PrisonersNSEP_MST_ID = '635'
IC_IV_STI_Treatment_MST_ID = '636'
IC_IV_Condoms_MST_ID = '140'
IC_IV_AGYW_PrEP_MST_ID = '637'
IC_IV_YouthFocIntervOutSch_MST_ID = '145'
IC_IV_MaleCircumcision_MST_ID = '141'
IC_IV_AdultFemalePrEP_MST_ID = '638'
IC_IV_AdultMalePrEP_MST_ID = '639'
IC_IV_PostExposProphylaxis_MST_ID = '151'
IC_IV_VolCounsTest_MST_ID = '139'
IC_IV_ART_M_MST_ID = '240'
IC_IV_ART_W_MST_ID = '241'
IC_IV_EconEmpowerCashTransfers_MST_ID = '640'
IC_IV_ScreenRiskCVD_Diab_MST_ID = '199'
IC_IV_FollowUpCareLR_CVD_Diab_MST_ID = '200'
IC_IV_TrHighCholLR_CVD_Diab_MST_ID = '201'
IC_IV_TrHighBloodPresLR_CVD_Diab_MST_ID = '202'
IC_IV_TrCVD_DiabRisk20To30_MST_ID = '203'
IC_IV_TrCVD_DiabRiskGT30_MST_ID = '204'
IC_IV_TrNewCasesAMI_Aspirin_MST_ID = '205'
IC_IV_TrIHD_PostMI_MST_ID = '206'
IC_IV_TrCerebrovascDisPostStroke_MST_ID = '207'
IC_IV_TrRheumHrtDisBenzPenic_MST_ID = '208'
IC_IV_StdGlycCtrl_MST_ID = '344'
IC_IV_IntensGlycCtrl_MST_ID = '345'
IC_IV_RetinopScreenPhotocoag_MST_ID = '346'
IC_IV_NeuropScreenPrevFootCare_MST_ID = '347'
IC_IV_ElimIndustrialTransFats_MST_ID = '485'
IC_IV_BasicBreastCancerAware_MST_ID = '225'
IC_IV_ScrClinBreastExam_MST_ID = '120'
IC_IV_ScrMammography_MST_ID = '226'
IC_IV_DiagScrClinBreastExam_MST_ID = '442'
IC_IV_DiagScrMammogram_MST_ID = '443'
IC_IV_DiagNoScrBreastCancer_MST_ID = '444'
IC_IV_BreastCancerTrS1_MST_ID = '227'
IC_IV_BreastCancerTrS2_MST_ID = '228'
IC_IV_BreastCancerTrS3_MST_ID = '229'
IC_IV_BreastCancerTrS4_MST_ID = '230'
IC_IV_Trastuzamab_MST_ID = '445'
IC_IV_PostTrSurvBreastCancerPat_MST_ID = '236'
IC_IV_BasicPallCareBreastCancer_MST_ID = '231'
IC_IV_ExtendPallCareBreastCancer_MST_ID = '232'
IC_IV_HPV_DNATest_MST_ID = '413'
IC_IV_VisualInspectAceticAcid_MST_ID = '414'
IC_IV_PapTest_MST_ID = '415'
IC_IV_HPV_DNA_VIA_MST_ID = '416'
IC_IV_HPV_DNA_PapSmear_MST_ID = '417'
IC_IV_BiopsyHistopathology_MST_ID = '418'
IC_IV_Cryotherapy_MST_ID = '419'
IC_IV_LEEP_MST_ID = '420'
IC_IV_CervCancerTrS1_MST_ID = '213'
IC_IV_CervCancerTrS2_MST_ID = '421'
IC_IV_CervCancerTrS3_MST_ID = '422'
IC_IV_CervCancerTrS4_MST_ID = '423'
IC_IV_PostTrSurvCervCancer_MST_ID = '424'
IC_IV_BasicPallCareCervCancer_MST_ID = '425'
IC_IV_ExtendPallCareCervCancer_MST_ID = '426'
IC_IV_ScrFecalImmChemTest_MST_ID = '427'
IC_IV_ScrFecalOccultBloodTest_MST_ID = '428'
IC_IV_ScrSigmoidoscopy_MST_ID = '429'
IC_IV_ScrColonoscopy_MST_ID = '430'
IC_IV_DiagColorCancerScrFIT_MST_ID = '431'
IC_IV_DiagColorCancerScrFOBT_MST_ID = '432'
IC_IV_DiagColorCancerScrSigm_MST_ID = '433'
IC_IV_DiagNoScrColorCancer_MST_ID = '434'
IC_IV_ColorCancerTrS1_MST_ID = '435'
IC_IV_ColorCancerTrS2_MST_ID = '436'
IC_IV_ColorCancerTrS3_MST_ID = '437'
IC_IV_ColorCancerTrS4_MST_ID = '438'
IC_IV_PostTrSurvColorCancer_MST_ID = '439'
IC_IV_BasicPallCareColorCancer_MST_ID = '440'
IC_IV_ExtendPallCareColorCancer_MST_ID = '441'
IC_IV_DentalCleanPrevCare_MST_ID = '218'
IC_IV_OralCancerTr_MST_ID = '221'
IC_IV_AsthmaSABA_Interm_MST_ID = '332'
IC_IV_AsthmaLDBeclSABA_MST_ID = '333'
IC_IV_AsthmaHDBeclSABA_MST_ID = '334'
IC_IV_AsthmaTheoHDBeclSABA_MST_ID = '335'
IC_IV_AsthmaPrednTheoHDBeclSABA_MST_ID = '336'
IC_IV_COPD_SmokingCessation_MST_ID = '337'
IC_IV_COPD_InhSalbut_MST_ID = '338'
IC_IV_COPD_LDTheo_MST_ID = '339'
IC_IV_COPD_IpraInhaler_MST_ID = '340'
IC_IV_COPD_ExcacerbTrAntib_MST_ID = '341'
IC_IV_COPD_ExcacerbTrPredn_MST_ID = '342'
IC_IV_COPD_ExcacerbTrOxy_MST_ID = '343'
IC_IV_AvgAnnEmerCareNeeds_MST_ID = '224'
IC_IV_RFsHelpQuitTobUse_MST_ID = '387'
IC_IV_RFsScrBriefIntervAlc_MST_ID = '388'
IC_IV_RFsPhysInactBriefAdvice_MST_ID = '389'
IC_IV_TobMonUsePrevPolicies_MST_ID = '390'
IC_IV_TobSmokeProtection_MST_ID = '391'
IC_IV_TobHelpToQuitMCess_MST_ID = '392'
IC_IV_TobWarningLabels_MST_ID = '393'
IC_IV_TobWarnViaMassMedia_MST_ID = '394'
IC_IV_TobEnforceBansAdvertis_MST_ID = '395'
IC_IV_TobEnforceYouthRestr_MST_ID = '396'
IC_IV_TobRaiseTaxes_MST_ID = '397'
IC_IV_TobPlainPackaging_MST_ID = '398'
IC_IV_AlcEnforceAvail_MST_ID = '399'
IC_IV_AlcEnforceRestrictAdvertis_MST_ID = '400'
IC_IV_AlcEnforceDrunkDrivLaws_MST_ID = '401'
IC_IV_AlcRaiseTaxes_MST_ID = '402'
IC_IV_PhysInactAwareCamp_MST_ID = '403'
IC_IV_NaSurv_MST_ID = '404'
IC_IV_NaIndustryReform_MST_ID = '405'
IC_IV_NaStdsFrontPackLab_MST_ID = '406'
IC_IV_NaStdsStratMisleadMarketing_MST_ID = '407'
IC_IV_NaEduComm_MST_ID = '408'
IC_IV_NaReductCommSpaces_MST_ID = '409'
IC_IV_ObesNoTransFats_MST_ID = '410'
IC_IV_ObesReplaceSaturFats_MST_ID = '411'
IC_IV_ObesTaxSugaryDrinks_MST_ID = '412'
IC_IV_AnxBasicPsychTrMild_MST_ID = '328'
IC_IV_AnxBasicPsychTrMed_MST_ID = '329'
IC_IV_AnxIntensPsychTrMed_MST_ID = '330'
IC_IV_DeprBasicPsychTrMild_MST_ID = '325'
IC_IV_DeprBasicPsychTrMed_MST_ID = '288'
IC_IV_DeprIntensPsychTrMed_MST_ID = '289'
IC_IV_DeprIntensPsychTrMedEpB_MST_ID = '348'
IC_IV_DeprIntensPsychTrMedMnB_MST_ID = '349'
IC_IV_DeprPsychCarePerinatal_MST_ID = '260'
IC_IV_PsychBasicPsychSuppMed_MST_ID = '290'
IC_IV_PsychIntensPsychSuppMed_MST_ID = '291'
IC_IV_BipDisBasicPsychTrMed_MST_ID = '264'
IC_IV_BipDisIntensPsychIntervMed_MST_ID = '267'
IC_IV_EpilBasicPsychSuppMed_MST_ID = '292'
IC_IV_DevDisBasicPsychTr_MST_ID = '270'
IC_IV_DevDisIntensPsychInterv_MST_ID = '271'
IC_IV_BehDisBasicPsychTr_MST_ID = '272'
IC_IV_BehDisIntensPsychInterv_MST_ID = '273'
IC_IV_FamPsychADHD_MST_ID = '326'
IC_IV_FamPsychCondDis_MST_ID = '327'
IC_IV_MethylphenidateMed_MST_ID = '274'
IC_IV_DemAssessDiagFollowUp_MST_ID = '275'
IC_IV_DemPharmTr_MST_ID = '276'
IC_IV_AlcIdAssessNewCases_MST_ID = '277'
IC_IV_AlcBriefIntervFollowUp_MST_ID = '278'
IC_IV_MgmtAlcWd_MST_ID = '279'
IC_IV_AlcRelapsePrevMed_MST_ID = '280'
IC_IV_DrugIdAssessNewCases_MST_ID = '281'
IC_IV_DrugBriefIntervFollowUp_MST_ID = '282'
IC_IV_MgmtOpioidWd_MST_ID = '283'
IC_IV_MgmtNonOpioidWd_MST_ID = '284'
IC_IV_SelfHarmAssessCare_MST_ID = '285'
IC_IV_SelfHarmBasicPsychTr_MST_ID = '286'
IC_IV_PestIntoxMgmt_MST_ID = '287'
IC_IV_VisionScr_MST_ID = '447'
IC_IV_TdVaccCoDelivery_MST_ID = '448'
IC_IV_InjuryCareIPVPhysAssault_MST_ID = '449'
IC_IV_InjuryCareIPVSexAssault_MST_ID = '450'
IC_IV_ContracNotInUnionLT15_MST_ID = '451'
IC_IV_MgmtCondPrimaryCare_MST_ID = '452'
IC_IV_SchistCh_MST_ID = '455'
IC_IV_SchistAd_MST_ID = '456'
IC_IV_SoilTransHelm_MST_ID = '457'
IC_IV_Onchocer_MST_ID = '458'
IC_IV_LymphFilar_MST_ID = '459'
IC_IV_Trachoma_MST_ID = '460'
IC_IV_YawsTCT_MST_ID = '461'
IC_IV_BuruliUlcer_MST_ID = '462'
IC_IV_CutanLeishman_MST_ID = '463'
IC_IV_VisceralLeishman_MST_ID = '464'
IC_IV_HumanAfricTrypan_MST_ID = '465'
IC_IV_Leprosy_MST_ID = '466'
IC_IV_Echinoc_MST_ID = '467'
IC_IV_ChagasDisease_MST_ID = '468'
IC_IV_LymphFilarHydroSurgery_MST_ID = '469'
IC_IV_Trichiasis_MST_ID = '470'
IC_IV_LymphedemaMgmt_MST_ID = '471'
IC_IV_Yaws_MST_ID = '472'
IC_IV_AedesAegMosquito_MST_ID = '473'
IC_IV_Blackflies_MST_ID = '474'
IC_IV_Sandflies_MST_ID = '475'
IC_IV_TriatomineBugs_MST_ID = '476'
IC_IV_RabiesVaccCanine_MST_ID = '477'
IC_IV_RabiesPostExpProph_MST_ID = '478'
IC_IV_ScreenProvInitPLHIV_GTE10YrsOnART_NotIll_MST_ID = '641'
IC_IV_ScreenProvInitPLHIV_Children0t9_MST_ID = '551'
IC_IV_ScreenProvInitPLHIV_GTE10YrsInpat_MST_ID = '550'
IC_IV_ScreenProvInitPLHIV_Children0t9Inpat_MST_ID = '674'
IC_IV_ScreenProvInitHHAdults15Plus_MST_ID = '555'
IC_IV_ScreenProvInitHHChildren0t14_MST_ID = '552'
IC_IV_ScreenProvInitHHChildren0t4_MST_ID = '553'
IC_IV_ScreenProvInitHHChildren5t14_MST_ID = '554'
IC_IV_ScreenProvInitPrisoners_MST_ID = '556'
IC_IV_ScreenProvInitMiners_MST_ID = '557'
IC_IV_ScreenProvInitFibroticLesions_MST_ID = '558'
IC_IV_ScreenProvInitRiskFactorsGTE_P1PercTBPrev_MST_ID = '559'
IC_IV_ScreenProvInitRiskFactorsLmtdHlthCare_MST_ID = '560'
IC_IV_ScreenProvInitGenPop_MST_ID = '561'
IC_IV_TBCaseFindPatInitAdults_MST_ID = '675'
IC_IV_TBCaseFindPatInitChildren_MST_ID = '676'
IC_IV_DiagPatInitPulmTBAdults_MST_ID = '574'
IC_IV_DiagPatInitPulmTBChildren_MST_ID = '576'
IC_IV_DiagPatInitExtraPulmTBAdults_MST_ID = '575'
IC_IV_DiagPatInitExtraPulmTBChildren_MST_ID = '577'
IC_IV_DiagProvInitPulmTB_PLHIV_Outpat_MST_ID = '580'
IC_IV_DiagProvInitPulmTB_PLHIV_Inpat_MST_ID = '581'
IC_IV_DiagProvInitPulmTB_PLHIV_ChildrenOutpat_MST_ID = '582'
IC_IV_DiagProvInitPulmTB_PLHIV_ChildrenInpat_MST_ID = '583'
IC_IV_DiagProvInitPulmTB_HHChildren_MST_ID = '579'
IC_IV_DiagProvInitPulmTBAllHRGsAdults_MST_ID = '578'
IC_IV_RifampIsonResistTest_MST_ID = '584'
IC_IV_IsonResistTestRifampSensCases_MST_ID = '585'
IC_IV_FluoroqResistTestRifampResistCases_MST_ID = '586'
IC_IV_RESIST_TEST_BEDAQ_LINEZ_LEVO_MOXI_MST_ID = '649'
IC_IV_TreatDrugSuscTBRifampIsonSensAdults_MST_ID = '597'
IC_IV_TreatDrugSuscTBRifampIsonSensChildren0t14_MST_ID = '598'
IC_IV_TreatDrugSuscTBRifampIsonSensChildren3MoTo16_MST_ID = '599'
IC_IV_TreatTBMeningPericardAdults_MST_ID = '600'
IC_IV_TreatDrugSuscTBMeningPericardChildren_MST_ID = '601'
IC_IV_NonMedTreatSuppPatFirstLine_MST_ID = '677'
IC_IV_TreatDrugResistTBRifampSensIsonResistAdults_MST_ID = '602'
IC_IV_TreatDrugResistTBRifampSensIsonResistChildren_MST_ID = '603'
IC_IV_BpalMOralBedaqRegDrugResistTBAdults_MST_ID = '604'
IC_IV_OralBedaqRegDrugResistTBChildren_MST_ID = '605'
IC_IV_BPalRegDrugResistTBAdults_MST_ID = '606'
IC_IV_LongerRegDrugResistTBTreatAdults_MST_ID = '607'
IC_IV_LongerRegDrugResistTBTreatChildren_MST_ID = '608'
IC_IV_TreatDrugResistTB_XDR_Adults_MST_ID = '609'
IC_IV_TreatDrugResistTB_XDR_Children_MST_ID = '610'
IC_IV_NonMedTreatSuppPatSecLine_MST_ID = '678'
IC_IV_TestLatentTB_HHContactAdults15Plus_MST_ID = '563'
IC_IV_TestLatentTB_HHContactChildren5t14_MST_ID = '562'
IC_IV_TestLatentTB_HHContactChildren0t4_MST_ID = '564'
IC_IV_TestLatentTBOtherHRGsAdults15Plus_MST_ID = '565'
IC_IV_TB_ORAL_PREV_MED_ADULTS_15_PLUS_MST_ID = '650'
IC_IV_TB_ORAL_PREV_MED_CHILDREN_0_14_MST_ID = '567'
IC_IV_TBOralPrevMedWRifampDrugResist_MST_ID = '569'
IC_IV_LungResectionSurgery_MST_ID = '673'
IC_IV_PalliativeCare_MST_ID = '625'



from os import environ

from AvenirCommon.Database import GB_get_db_json

import SpectrumCommon.Const.GB as gbc

import SpectrumCommon.Const.IH.IHConstLink as ihcl

import SpectrumCommon.Const.IH.IHModVarFields as ihf
import SpectrumCommon.Modvars.IH.IHModVarDefs as ihmvd

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def IH_load_group_DB(costing_mode = int):
    environment = environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV]

    group_DB = []
    group_DB_IHT_ID_info_only = None

    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        group_DB_file_name = ihcl.IH_IV_GROUP_DB_NAME + '_OH_' + ihcl.IH_IV_GROUP_DB_CURR_VERSION + '.' + gbc.GB_JSON 
        group_DB = GB_get_db_json(environment, gbc.GB_IH_CORE_CONTAINER, group_DB_file_name) 

    # If costing mode was not specified, assume IHT since that is Intervention Costing's main purpose for existing.
    elif costing_mode == gbc.GB_TB_COSTING_MODE:
        group_DB_file_name = ihcl.IH_IV_GROUP_DB_NAME + '_TI_' + ihcl.IH_IV_TI_GROUP_DB_CURR_VERSION + '.' + gbc.GB_JSON 
        group_DB = GB_get_db_json(environment, gbc.GB_IH_CORE_CONTAINER, group_DB_file_name) 

        group_DB_IHT_ID_info_only_file_name = ihcl.IH_IV_GROUP_DB_NAME + '_OH_' + ihcl.IH_IV_TI_GROUP_DB_CURR_VERSION + '.' + gbc.GB_JSON 
        group_DB_IHT_ID_info_only = GB_get_db_json(environment, gbc.GB_IH_CORE_CONTAINER, group_DB_IHT_ID_info_only_file_name) 

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        group_DB_file_name = ihcl.IH_IV_GROUP_DB_NAME + '_' + ihcl.IH_IV_CS_GROUP_DB_CURR_VERSION + '.' + gbc.GB_JSON 
        group_DB = GB_get_db_json(environment, gbc.GB_CS_CONTAINER, group_DB_file_name) 

    return group_DB, group_DB_IHT_ID_info_only

#######################################################################################################
#                                                                                                     #
#                                       Programme areas and subgroups                                 #
#                                                                                                     #
#######################################################################################################

def IH_init_prog_area_records(group_DB):

    prog_area_records = []

    for group_DB_obj in group_DB:

        prog_area_mstID = str(group_DB_obj[ihcl.IH_IV_GROUP_MST_ID_KEY_GDB]) 
        subgroup_mstID = str(group_DB_obj[ihcl.IH_IV_SUBGROUP_MST_ID_KEY_GDB])
        english_str = group_DB_obj[ihcl.IH_IV_ENGLISH_STRING_KEY_GDB]
        str_constant = group_DB_obj[ihcl.IH_IV_STRING_CONSTANT_KEY_GDB]

        # Programme areas will have a subgroup master ID of -1. Programme areas should always come before their subgroups in the database.
        if subgroup_mstID == str(gbc.GB_NOT_FOUND):
            prog_area_rec = ihmvd.IH_create_prog_area_rec(prog_area_mstID, english_str, str_constant, True)
            prog_area_records.append(prog_area_rec)

        else:
            subgroup_rec = ihmvd.IH_create_prog_area_subgroup_rec(subgroup_mstID, english_str, str_constant, True)
            '''
                Since the database lists programme areas first, then their subgroups, then repeats, we should always be able to add the subgroup to 
                the current programme area.
            '''
            prog_area_records[-1][ihf.IH_PROG_AREA_SUBGROUP_RECORDS].append(subgroup_rec)
            

    return prog_area_records

# Initializes a list of programme area records containing only enough info to identify and display them in a list. Used for TB Costing to allow
# the users to pick IHT interventions and transfer them to TB standalone app.
def IH_init_prog_area_records_ID_info_only(group_DB):

    prog_area_records = []

    for group_DB_obj in group_DB:

        prog_area_mstID = str(group_DB_obj[ihcl.IH_IV_GROUP_MST_ID_KEY_GDB]) 
        subgroup_mstID = str(group_DB_obj[ihcl.IH_IV_SUBGROUP_MST_ID_KEY_GDB])
        english_str = group_DB_obj[ihcl.IH_IV_ENGLISH_STRING_KEY_GDB]
        str_constant = group_DB_obj[ihcl.IH_IV_STRING_CONSTANT_KEY_GDB]

        # Programme areas will have a subgroup master ID of -1. Programme areas should always come before their subgroups in the database.
        if subgroup_mstID == str(gbc.GB_NOT_FOUND):
            prog_area_rec = ihmvd.IH_create_prog_area_rec_ID_info_only(prog_area_mstID, english_str, str_constant)
            prog_area_records.append(prog_area_rec)

        else:
            subgroup_rec = ihmvd.IH_create_prog_area_subgroup_rec_ID_info_only(subgroup_mstID, english_str, str_constant)
            '''
                Since the database lists programme areas first, then their subgroups, then repeats, we should always be able to add the subgroup to 
                the current programme area.
            '''
            prog_area_records[-1][ihf.IH_PROG_AREA_SUBGROUP_RECORDS].append(subgroup_rec)

    return prog_area_records


#######################################################################################################
#                                                                                                     #
#                                       Delivery channels                                             #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       Delivery channel packages                                     #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       National programmes                                           #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       Audit records                                                 #
#                                                                                                     #
#######################################################################################################



from SpectrumCommon.Const.TB import *
def TB_screening_alg_patient_options():
    
    return [[TB_Scrn_g_sym_any_cxr_cad_parallel, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # PTB_HIVn_014             
            [TB_Scrn_g_sym_any_cxr_cad_parallel, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # PTB_HIVn_15p
            [TB_Scrn_h_clhiv_sym, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # PTB_HIVp_09 
            [TB_Scrn_g_sym_any_cxr_tb_parallel,	TB_Scrn_h_hiv_w4ss,	TB_Scrn_h_hiv_crp,	TB_Scrn_h_hiv_cxr_abn,
             TB_Scrn_h_hiv_w4ss_hiv_crp_parallel, 
             TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos, TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_neg, 
             TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos,	TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg,
             TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # PTB_HIVp_1014
            [TB_Scrn_g_sym_any_cxr_tb_parallel,	TB_Scrn_h_hiv_w4ss,	TB_Scrn_h_hiv_crp,	TB_Scrn_h_hiv_cxr_abn,
             TB_Scrn_h_hiv_w4ss_hiv_crp_parallel, 
             TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos, TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_neg, 
             TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos,	TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg,
             TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # PTB_HIVp_15p
            [TB_Scrn_g_sym_any, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ETB_HIVn_014
            [TB_Scrn_g_sym_any, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ETB_HIVn_15p
            [TB_Scrn_g_sym_any, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ETB_HIVp_09
            [TB_Scrn_g_sym_any, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ETB_HIVp_1014
            [TB_Scrn_g_sym_any, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ETB_HIVp_15p
    ]

def TB_screening_alg_patient_defaults():
    return [TB_Scrn_g_sym_any_cxr_cad_parallel, # PTB_HIVn_014             
            TB_Scrn_g_sym_any_cxr_cad_parallel, # PTB_HIVn_15p
            TB_Scrn_h_clhiv_sym, # PTB_HIVp_09 
            TB_Scrn_g_sym_any_cxr_tb_parallel, # PTB_HIVp_1014
            TB_Scrn_g_sym_any_cxr_tb_parallel, # PTB_HIVp_15p
            TB_Scrn_g_sym_any, # ETB_HIVn_014
            TB_Scrn_g_sym_any, # ETB_HIVn_15p
            TB_Scrn_g_sym_any, # ETB_HIVp_09
            TB_Scrn_g_sym_any, # ETB_HIVp_1014
            TB_Scrn_g_sym_any, # ETB_HIVp_15p
    ]

def TB_screening_alg_household_options():
    
    return [[TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel,
             TB_Scrn_c_cc_sym,  TB_Scrn_c_cc_cxr_tb, 
             TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_pos, TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_neg, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # HH 0-4
            [TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel,
             TB_Scrn_c_cc_sym,  TB_Scrn_c_cc_cxr_tb, 
             TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_pos, TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_neg, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # HH 5-14
            [TB_Scrn_g_sym_any_cxr_tb_parallel, TB_Scrn_g_sym_cgh2w, TB_Scrn_g_sym_cgh2w_cxr_cad_parallel,
             TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos,	TB_Scrn_g_sym_cgh2w,	TB_Scrn_g_sym_any,
             TB_Scrn_g_sym_any_cxr_tb_sequential_pos, TB_Scrn_g_sym_any_cxr_tb_sequential_neg,
             TB_Scrn_g_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom] # HH 15+
        ]

def TB_screening_alg_household_defaults():
    
    return [TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel, # HH 0-4
            TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel, # HH 5-14
            TB_Scrn_g_sym_any_cxr_tb_parallel,  # HH 15+
        ]



def TB_screening_alg_art_options():
    result = [[],[]] 
    art_not_serious = [[TB_Scrn_h_clhiv_sym, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ART 0-9
                       [TB_Scrn_h_hiv_w4ss_cxr_cad_parallel, TB_Scrn_h_hiv_w4ss, TB_Scrn_h_hiv_crp, TB_Scrn_h_hiv_cxr_abn,
                        TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos, TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_neg,
                        TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos, TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg, TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ART 10-14   
                       [TB_Scrn_h_hiv_w4ss_cxr_cad_parallel, TB_Scrn_h_hiv_w4ss, TB_Scrn_h_hiv_crp, TB_Scrn_h_hiv_cxr_abn,
                        TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos, TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_neg,
                        TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos, TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg, TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom]] # ART 15+

    art_serious = [[TB_Scrn_h_clhiv_sym, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ART 0-9
                   [TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom], # ART 10-14
                   [TB_Scrn_h_hiv_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom]] # ART 15+
    
    result[TB_ARTNotServere] = art_not_serious
    result[TB_ARTSevereIllness] = art_serious

    return result

def TB_screening_alg_art_defaults():
    result = [[],[]] 
    art_not_serious = [TB_Scrn_h_clhiv_sym, # ART 0-9
                       TB_Scrn_h_hiv_w4ss_cxr_cad_parallel, # ART 10-14   
                       TB_Scrn_h_hiv_w4ss_cxr_cad_parallel] # ART 15+

    art_serious = [TB_Scrn_h_clhiv_sym, # ART 0-9
                   TB_Scrn_h_hiv_mwrd_sn, # ART 10-14
                   TB_Scrn_h_hiv_mwrd_sn] # ART 15+
    
    result[TB_ARTNotServere] = art_not_serious
    result[TB_ARTSevereIllness] = art_serious

    return result


def TB_screening_alg_hrg_options():

    prisoners  = [TB_Scrn_g_sym_any_cxr_tb_parallel, TB_Scrn_g_sym_cgh2w, TB_Scrn_g_sym_cgh2w_cxr_tb_parallel, TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos,
                  TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_neg, TB_Scrn_g_sym_any, TB_Scrn_g_sym_any_cxr_tb_sequential_pos, 
                  TB_Scrn_g_sym_any_cxr_tb_sequential_neg, TB_Scrn_g_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom]
    other_hrgs = [TB_Scrn_g_sym_any_cxr_tb_parallel, TB_Scrn_g_sym_cgh2w, TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos,	
                  TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_neg,	TB_Scrn_g_sym_any,
                  TB_Scrn_g_sym_any_cxr_tb_parallel,	TB_Scrn_g_sym_any_cxr_tb_sequential_pos,	TB_Scrn_g_sym_any_cxr_tb_sequential_neg,	TB_Scrn_g_mwrd_sn, TB_Scrn_1Step_custom, TB_Scrn_2Step_custom]
     
    result = [other_hrgs for hrg in TB_HRG_List] # default everything to other
    result[TB_HRG_Prisoners]  = prisoners

    return result

def TB_screening_alg_hrg_defaults():

    prisoners  = TB_Scrn_g_sym_any_cxr_tb_parallel
    other_hrgs = TB_Scrn_g_sym_any_cxr_tb_parallel
     
    result = [other_hrgs for hrg in TB_HRG_List] # default everything to other
    result[TB_HRG_Prisoners] = prisoners

    return result



def TB_screening_sens_spec_combined(screenID):
    screen_map = TB_screening_sens_spec_map()
    return screen_map[screenID]

def TB_screening_sens_spec_map():
    return {
        TB_Scrn_g_sym_cgh2w                           : (0.420, 0.940),           
        TB_Scrn_g_sym_any                             : (0.710, 0.640),           
        TB_Scrn_g_mwrd_sn                             : (0.690, 0.990),           
        TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_pos     : (0.357, 0.998),           
        TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_neg     : (0.357, 0.902),           
        TB_Scrn_g_sym_cgh2w_cxr_tb_parallel           : (0.913, 0.902),           
        TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos    : (0.395, 0.993),           
        TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_neg    : (0.395, 0.837),           
        TB_Scrn_g_sym_cgh2w_cxr_cad_parallel          : (0.965, 0.837),           
        TB_Scrn_g_sym_any_cxr_tb_sequential_pos       : (0.604, 0.986),           
        TB_Scrn_g_sym_any_cxr_tb_sequential_neg       : (0.604, 0.614),           
        TB_Scrn_g_sym_any_cxr_tb_parallel             : (0.957, 0.614),           
        TB_Scrn_g_sym_any_cxr_cad_sequential_pos      : (0.667, 0.960),           
        TB_Scrn_g_sym_any_cxr_cad_sequential_neg      : (0.667, 0.570),           
        TB_Scrn_g_sym_any_cxr_cad_parallel            : (0.983, 0.570),           
        TB_Scrn_h_clhiv_sym                           : (0.610, 0.940),           
        TB_Scrn_h_hiv_mwrd_sn                         : (0.690, 0.980),           
        TB_Scrn_h_hiv_w4ss                            : (0.830, 0.380),           
        TB_Scrn_h_hiv_crp                             : (0.900, 0.500),           
        TB_Scrn_h_hiv_cxr_abn                         : (0.930, 0.200),           
        TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos     : (0.747, 0.690),           
        TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg     : (0.747, 0.190),           
        TB_Scrn_h_hiv_w4ss_hiv_crp_parallel           : (0.983, 0.190),           
        TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos : (0.772, 0.504),           
        TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_neg : (0.772, 0.076),           
        TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_parallel       : (0.988, 0.076),           
        TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_pos     : (0.780, 0.932),           
        TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_neg     : (0.780, 0.338),           
        TB_Scrn_h_hiv_w4ss_cxr_cad_parallel           : (0.990, 0.338),           
        TB_Scrn_c_cc_sym                              : (0.890, 0.690),           
        TB_Scrn_c_cc_cxr_tb                           : (0.840, 0.910),           
        TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_pos   : (0.697, 0.944),           
        TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_neg   : (0.697, 0.346),           
        TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel         : (0.973, 0.346),           
        TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_pos     : (0.780, 0.932),           
        TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_neg     : (0.780, 0.338),           
        TB_Scrn_c_hiv_w4ss_cxr_cad_parallel           : (0.990, 0.338),           
    }





























	






















































CS_IVG_Periconceptual_MST_ID = '1'
CS_IVG_Pregnancy_MST_ID = '4'
CS_IVG_Childbirth_MST_ID = '7'
CS_IVG_Breastfeeding_MST_ID = '13'
CS_IVG_PostnatalPreventive_MST_ID = '14'
CS_IVG_Vaccines_MST_ID = '15'
CS_IVG_PostnatalCurative_MST_ID = '16'
CS_GROUP_MST_IDS = [CS_IVG_Periconceptual_MST_ID, CS_IVG_Pregnancy_MST_ID, CS_IVG_Childbirth_MST_ID, CS_IVG_Breastfeeding_MST_ID, CS_IVG_PostnatalPreventive_MST_ID, CS_IVG_Vaccines_MST_ID, CS_IVG_PostnatalCurative_MST_ID]


import numpy as np

def alloc_parameters(x, f_r, f_p, f_xi):
    
    r = f_r.copy()
    p = f_p.copy()
    xi = f_xi.copy()
    r['beta'] = x[xi['r_beta']] #r.beta         = x(xi.r_beta); #Transmission for HIV-ve, D3S-TB
    p['rfbeta_mdr'] = x[xi['rfbeta_mdr']] #p.rfbeta_mdr   = x(xi.rfbeta_mdr);
    p['rfbeta_hiv'] = x[xi['rfbeta_hiv']] #p.rfbeta_hiv   = x(xi.rfbeta_hiv);
    r['betared']      = x[xi['r_betared']] #r.betared      = x(xi.r_betared); #New, added on 2024-04-02
    r['r_sym'] = x[xi['r_sym']] # r.sym          = x(xi.r_sym);
    p['pu_2022'] = x[xi['p_pu']] #p['pu'] = x[xi['p_pu']] #p.pu           = x(xi.p_pu);
    #r['cs'] = x[xi['r_cs']]*np.array([1,1,1,1,1]) # r.cs           = x(xi.r_cs);
    #r['cs2'] = x[xi['r_cs2']]*np.array([1,1,1,1,1]) #r.cs2          = x(xi.r_cs2);
    #r['cs3'] = x[xi['r_cs3']]*[1,1,1,1,1] #r.cs2          = x(xi.r_cs2);
    ##
    r['cs_1997'] = x[xi['r_cs']]*np.array([1,1,1,1,1]) #r.cs_1997      = x(xi.r_cs).*[1,1,1,1,1];
    r['cs_2022'] = x[xi['r_cs']]*x[xi['rf_cs2022']]*np.array([1,1,1,1,1]) #r.cs_2022      = x(xi.r_cs)*x(xi.rf_cs2022).*[1,1,1,1,1];
    r['cs2_1997'] = x[xi['r_cs']]*x[xi['rf_cs2']]*np.array([1,1,1,1,1]) #r.cs2_1997     = x(xi.r_cs)*x(xi.rf_cs2).*[1,1,1,1,1];
    r['cs2_2022'] = x[xi['r_cs']]*x[xi['rf_cs2']]*np.array([1,1,1,1,1]) #r.cs2_2022     = x(xi.r_cs)*x(xi.rf_cs2).*[1,1,1,1,1];
    ##
    r['cs3'] = r['cs3']*np.array([1,1,1,1,1]) #r.cs2          = x(xi.r_cs2);
    #p['Tx_init'] = x[xi['Tx_init']]
    r['mort_TB'] = x[xi['r_mort_TB']] #r.mort_TB      = x(xi.r_mort_TB);
    p['Dx'] = x[xi['p_Dx']] #p.Dx           = x(xi.p_Dx);
    r['default'] = r['Tx']*(1-x[xi['p_Tx_complete']])/x[xi['p_Tx_complete']] #r.default      = r.Tx*(1-x(xi.p_Tx_complete))./x(xi.p_Tx_complete);
   
    #r['HIV_acqu'] = x[xi['r_HIV_acqu']] #% r.HIV_acqu = x(xi.r_HIV_acqu);
    r['ART_init'] = x[xi['r_ART_init']] #r.ART_init     = x(xi.r_ART_init);

    #p['MDR_rec'] = [x[xi['p_MDRrec2019']],0] #p.MDR_rec      = [x(xi.p_MDRrec2019),0]; Modified after Sandip changes on 2024-04-02 (next line)
    p['MDR_rec']      = [x[xi['p_MDRrec2022']],0] #p.MDR_rec      = [x(xi.p_MDRrec2022),0];
    r['MDR_acqu'] = x[xi['r_MDR_acqu']] #r.MDR_acqu     = x(xi.r_MDR_acqu);
    r['HIV_mort'] = x[xi['r_HIV_mort']] #r.HIV_mort     = x(xi.r_HIV_mort);
    
    r['progression'] = np.array(r['progression'])
    r['reactivation'] = np.array(r['reactivation'])
    r['progression'][[1,2]] = r['progression'][0]*x[xi['p_HIV_relrate']]*np.array([1, 0.4]).T #r.progression([2,3])  = r.progression(1)*x(xi.p_HIV_relrate)*[1 0.4];
    r['reactivation'][[1,2]] = r['reactivation'][0]*x[xi['p_HIV_relrate']]*np.array([1, 0.4]) #r.reactivation([2,3]) = r.reactivation(1)*x(xi.p_HIV_relrate)*[1 0.4];
   
    #New lines
    #r['progression']   = np.tile(r['progression'],[5,1])#repmat(r.progression,[5,1]); #%[0.0826    3.4262    1.3705].*[1 1 1 1 1]'; %
    #r['reactivation']  = np.tile(r['reactivation'],[5,1])#repmat(r.reactivation,[5,1]); #%[0.0006    0.0249    0.0100].*[1 1 1 1 1]'; %
   
    r['progression']   = (np.reshape(np.array(r['progression']),(3,1))@np.reshape(np.array([1, 1, 1, 1, 1]),(1,5))).T #;    %[0.0826    3.4262    1.3705].*[1 1 1 1 1]'; %
    r['reactivation']  = (np.reshape(np.array(r['reactivation']),(3,1))@np.reshape(np.array([1, 1, 1, 1, 1]),(1,5))).T #r.reactivation  = r.reactivation.*[1 1 1 1 1]';   %[0.0006    0.0249    0.0100].*[1 1 1 1 1]'; %

    #if len(x)>xi['calib']:
    #    r['self_cure']   = x[xi['r_self_cure']]
    r['self_cure'][[0,2]]   = x[xi['r_self_cure']]
    return (r, p)
